# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Network`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Network/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from . import io
from . import page
from . import runtime
from . import security
from .utils import memoize_event


class ResourceType(str, enum.Enum):
    """Resource type as it was perceived by the rendering engine."""

    DOCUMENT = "document"
    STYLESHEET = "stylesheet"
    IMAGE = "image"
    MEDIA = "media"
    FONT = "font"
    SCRIPT = "script"
    TEXT_TRACK = "text_track"
    X_H_R = "xhr"
    FETCH = "fetch"
    PREFETCH = "prefetch"
    EVENT_SOURCE = "event_source"
    WEB_SOCKET = "web_socket"
    MANIFEST = "manifest"
    SIGNED_EXCHANGE = "signed_exchange"
    PING = "ping"
    C_S_P_VIOLATION_REPORT = "csp_violation_report"
    PREFLIGHT = "preflight"
    OTHER = "other"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class LoaderId(str):
    """Unique loader identifier."""

    def to_json(self) -> LoaderId:
        return self

    @classmethod
    def from_json(cls, value: str) -> LoaderId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class RequestId(str):
    """Unique request identifier."""

    def to_json(self) -> RequestId:
        return self

    @classmethod
    def from_json(cls, value: str) -> RequestId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class InterceptionId(str):
    """Unique intercepted request identifier."""

    def to_json(self) -> InterceptionId:
        return self

    @classmethod
    def from_json(cls, value: str) -> InterceptionId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class ErrorReason(str, enum.Enum):
    """Network level fetch failure reason."""

    FAILED = "failed"
    ABORTED = "aborted"
    TIMED_OUT = "timed_out"
    ACCESS_DENIED = "access_denied"
    CONNECTION_CLOSED = "connection_closed"
    CONNECTION_RESET = "connection_reset"
    CONNECTION_REFUSED = "connection_refused"
    CONNECTION_ABORTED = "connection_aborted"
    CONNECTION_FAILED = "connection_failed"
    NAME_NOT_RESOLVED = "name_not_resolved"
    INTERNET_DISCONNECTED = "internet_disconnected"
    ADDRESS_UNREACHABLE = "address_unreachable"
    BLOCKED_BY_CLIENT = "blocked_by_client"
    BLOCKED_BY_RESPONSE = "blocked_by_response"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class TimeSinceEpoch(float):
    """UTC time in seconds, counted from January 1, 1970."""

    def to_json(self) -> TimeSinceEpoch:
        return self

    @classmethod
    def from_json(cls, value: float) -> TimeSinceEpoch:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class MonotonicTime(float):
    """Monotonically increasing time in seconds since an arbitrary point in the past."""

    def to_json(self) -> MonotonicTime:
        return self

    @classmethod
    def from_json(cls, value: float) -> MonotonicTime:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class Headers:
    """Request / response headers as keys / values of JSON object."""


class ConnectionType(str, enum.Enum):
    """The underlying connection technology that the browser is supposedly using."""

    NONE = "none"
    CELLULAR2G = "cellular2g"
    CELLULAR3G = "cellular3g"
    CELLULAR4G = "cellular4g"
    BLUETOOTH = "bluetooth"
    ETHERNET = "ethernet"
    WIFI = "wifi"
    WIMAX = "wimax"
    OTHER = "other"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class CookieSameSite(str, enum.Enum):
    """Represents the cookie's 'SameSite' status:

    https://tools.ietf.org/html/draft-west-first-party-cookies
    """

    STRICT = "strict"
    LAX = "lax"
    NONE = "none"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class CookiePriority(str, enum.Enum):
    """Represents the cookie's 'Priority' status:

    https://tools.ietf.org/html/draft-west-cookie-priority-00
    """

    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class CookieSourceScheme(str, enum.Enum):
    """Represents the source scheme of the origin that originally set the cookie.

    A value of "Unset" allows protocol clients to emulate legacy cookie scope for the scheme. This is a temporary
    ability and it will be removed in the future.
    """

    UNSET = "unset"
    NON_SECURE = "non_secure"
    SECURE = "secure"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class ResourceTiming:
    """Timing information for the request."""

    # Timing's requestTime is a baseline in seconds, while the other numbers areticks in milliseconds relatively to this requestTime. # noqa
    request_time: float
    # Started resolving proxy. # noqa
    proxy_start: float
    # Finished resolving proxy. # noqa
    proxy_end: float
    # Started DNS address resolve. # noqa
    dns_start: float
    # Finished DNS address resolve. # noqa
    dns_end: float
    # Started connecting to the remote host. # noqa
    connect_start: float
    # Connected to the remote host. # noqa
    connect_end: float
    # Started SSL handshake. # noqa
    ssl_start: float
    # Finished SSL handshake. # noqa
    ssl_end: float
    # Started running ServiceWorker. # noqa
    worker_start: float
    # Finished Starting ServiceWorker. # noqa
    worker_ready: float
    # Started fetch event. # noqa
    worker_fetch_start: float
    # Settled fetch event respondWith promise. # noqa
    worker_respond_with_settled: float
    # Started sending request. # noqa
    send_start: float
    # Finished sending request. # noqa
    send_end: float
    # Time the server started pushing request. # noqa
    push_start: float
    # Time the server finished pushing request. # noqa
    push_end: float
    # Finished receiving response headers. # noqa
    receive_headers_end: float


class ResourcePriority(str, enum.Enum):
    """Loading priority of a resource request."""

    VERY_LOW = "very_low"
    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"
    VERY_HIGH = "very_high"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class PostDataEntry:
    """Post data entry for HTTP request."""

    # Description is missing from the devtools protocol document. # noqa
    bytes: typing.Optional[str]


@dataclass
class Request:
    """HTTP request data."""

    # Request URL (without fragment). # noqa
    url: str
    # HTTP request method. # noqa
    method: str
    # HTTP request headers. # noqa
    headers: Headers
    # Priority of the resource request at the time request is sent. # noqa
    initial_priority: ResourcePriority
    # The referrer policy of the request, as defined inhttps://www.w3.org/TR/referrer-policy/ # noqa
    referrer_policy: typing.List[
        typing.Literal[
            "unsafe-url",
            "no-referrer-when-downgrade",
            "no-referrer",
            "origin",
            "origin-when-cross-origin",
            "same-origin",
            "strict-origin",
            "strict-origin-when-cross-origin",
        ]
    ]
    # Fragment of the requested URL starting with hash, if present. # noqa
    url_fragment: typing.Optional[str]
    # HTTP POST request data. # noqa
    post_data: typing.Optional[str]
    # True when the request has POST data. Note that postData might still beomitted when this flag is true when the data is too long. # noqa
    has_post_data: typing.Optional[bool]
    # Request body elements. This will be converted from base64 to binary # noqa
    post_data_entries: typing.Optional[PostDataEntry]
    # The mixed content type of the request. # noqa
    mixed_content_type: typing.Optional[security.MixedContentType]
    # Whether is loaded via link preload. # noqa
    is_link_preload: typing.Optional[bool]
    # Set for requests when the TrustToken API is used. Contains the parameterspassed by the developer (e.g. via "fetch") as understood by the backend. # noqa
    trust_token_params: typing.Optional[TrustTokenParams]
    # True if this resource request is considered to be the 'same site' as therequest correspondinfg to the main frame. # noqa
    is_same_site: typing.Optional[bool]


@dataclass
class SignedCertificateTimestamp:
    """Details of a signed certificate timestamp (SCT)."""

    # Validation status. # noqa
    status: str
    # Origin. # noqa
    origin: str
    # Log name / description. # noqa
    log_description: str
    # Log ID. # noqa
    log_id: str
    # Issuance date. Unlike TimeSinceEpoch, this contains the number ofmilliseconds since January 1, 1970, UTC, not the number of seconds. # noqa
    timestamp: float
    # Hash algorithm. # noqa
    hash_algorithm: str
    # Signature algorithm. # noqa
    signature_algorithm: str
    # Signature data. # noqa
    signature_data: str


@dataclass
class SecurityDetails:
    """Security details about a request."""

    # Protocol name (e.g. "TLS 1.2" or "QUIC"). # noqa
    protocol: str
    # Key Exchange used by the connection, or the empty string if notapplicable. # noqa
    key_exchange: str
    # Cipher name. # noqa
    cipher: str
    # Certificate ID value. # noqa
    certificate_id: security.CertificateId
    # Certificate subject name. # noqa
    subject_name: str
    # Subject Alternative Name (SAN) DNS names and IP addresses. # noqa
    san_list: str
    # Name of the issuing CA. # noqa
    issuer: str
    # Certificate valid from date. # noqa
    valid_from: TimeSinceEpoch
    # Certificate valid to (expiration) date # noqa
    valid_to: TimeSinceEpoch
    # List of signed certificate timestamps (SCTs). # noqa
    signed_certificate_timestamp_list: SignedCertificateTimestamp
    # Whether the request complied with Certificate Transparency policy # noqa
    certificate_transparency_compliance: CertificateTransparencyCompliance
    # Whether the connection used Encrypted ClientHello # noqa
    encrypted_client_hello: bool
    # (EC)DH group used by the connection, if applicable. # noqa
    key_exchange_group: typing.Optional[str]
    # TLS MAC. Note that AEAD ciphers do not have separate MACs. # noqa
    mac: typing.Optional[str]
    # The signature algorithm used by the server in the TLS server signature,represented as a TLS SignatureScheme code point. Omitted if not applicable ornot known. # noqa
    server_signature_algorithm: typing.Optional[int]


class CertificateTransparencyCompliance(str, enum.Enum):
    """Whether the request complied with Certificate Transparency policy."""

    UNKNOWN = "unknown"
    NOT_COMPLIANT = "not-compliant"
    COMPLIANT = "compliant"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class BlockedReason(str, enum.Enum):
    """The reason why request was blocked."""

    OTHER = "other"
    CSP = "csp"
    MIXED_CONTENT = "mixed-content"
    ORIGIN = "origin"
    INSPECTOR = "inspector"
    SUBRESOURCE_FILTER = "subresource-filter"
    CONTENT_TYPE = "content-type"
    COEP_FRAME_RESOURCE_NEEDS_COEP_HEADER = "coep-frame-resource-needs-coep-header"
    COOP_SANDBOXED_IFRAME_CANNOT_NAVIGATE_TO_COOP_PAGE = "coop-sandboxed-iframe-cannot-navigate-to-coop-page"
    CORP_NOT_SAME_ORIGIN = "corp-not-same-origin"
    CORP_NOT_SAME_ORIGIN_AFTER_DEFAULTED_TO_SAME_ORIGIN_BY_COEP = (
        "corp-not-same-origin-after-defaulted-to-same-origin-by-coep"
    )
    CORP_NOT_SAME_SITE = "corp-not-same-site"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class CorsError(str, enum.Enum):
    """The reason why request was blocked."""

    DISALLOWED_BY_MODE = "disallowed_by_mode"
    INVALID_RESPONSE = "invalid_response"
    WILDCARD_ORIGIN_NOT_ALLOWED = "wildcard_origin_not_allowed"
    MISSING_ALLOW_ORIGIN_HEADER = "missing_allow_origin_header"
    MULTIPLE_ALLOW_ORIGIN_VALUES = "multiple_allow_origin_values"
    INVALID_ALLOW_ORIGIN_VALUE = "invalid_allow_origin_value"
    ALLOW_ORIGIN_MISMATCH = "allow_origin_mismatch"
    INVALID_ALLOW_CREDENTIALS = "invalid_allow_credentials"
    CORS_DISABLED_SCHEME = "cors_disabled_scheme"
    PREFLIGHT_INVALID_STATUS = "preflight_invalid_status"
    PREFLIGHT_DISALLOWED_REDIRECT = "preflight_disallowed_redirect"
    PREFLIGHT_WILDCARD_ORIGIN_NOT_ALLOWED = "preflight_wildcard_origin_not_allowed"
    PREFLIGHT_MISSING_ALLOW_ORIGIN_HEADER = "preflight_missing_allow_origin_header"
    PREFLIGHT_MULTIPLE_ALLOW_ORIGIN_VALUES = "preflight_multiple_allow_origin_values"
    PREFLIGHT_INVALID_ALLOW_ORIGIN_VALUE = "preflight_invalid_allow_origin_value"
    PREFLIGHT_ALLOW_ORIGIN_MISMATCH = "preflight_allow_origin_mismatch"
    PREFLIGHT_INVALID_ALLOW_CREDENTIALS = "preflight_invalid_allow_credentials"
    PREFLIGHT_MISSING_ALLOW_EXTERNAL = "preflight_missing_allow_external"
    PREFLIGHT_INVALID_ALLOW_EXTERNAL = "preflight_invalid_allow_external"
    PREFLIGHT_MISSING_ALLOW_PRIVATE_NETWORK = "preflight_missing_allow_private_network"
    PREFLIGHT_INVALID_ALLOW_PRIVATE_NETWORK = "preflight_invalid_allow_private_network"
    INVALID_ALLOW_METHODS_PREFLIGHT_RESPONSE = "invalid_allow_methods_preflight_response"
    INVALID_ALLOW_HEADERS_PREFLIGHT_RESPONSE = "invalid_allow_headers_preflight_response"
    METHOD_DISALLOWED_BY_PREFLIGHT_RESPONSE = "method_disallowed_by_preflight_response"
    HEADER_DISALLOWED_BY_PREFLIGHT_RESPONSE = "header_disallowed_by_preflight_response"
    REDIRECT_CONTAINS_CREDENTIALS = "redirect_contains_credentials"
    INSECURE_PRIVATE_NETWORK = "insecure_private_network"
    INVALID_PRIVATE_NETWORK_ACCESS = "invalid_private_network_access"
    UNEXPECTED_PRIVATE_NETWORK_ACCESS = "unexpected_private_network_access"
    NO_CORS_REDIRECT_MODE_NOT_FOLLOW = "no_cors_redirect_mode_not_follow"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class CorsErrorStatus:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    cors_error: CorsError
    # Description is missing from the devtools protocol document. # noqa
    failed_parameter: str


class ServiceWorkerResponseSource(str, enum.Enum):
    """Source of serviceworker response."""

    CACHE_STORAGE = "cache-storage"
    HTTP_CACHE = "http-cache"
    FALLBACK_CODE = "fallback-code"
    NETWORK = "network"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class TrustTokenParams:
    """Determines what type of Trust Token operation is executed and depending on the type, some additional parameters.

    The values
    are specified in third_party/blink/renderer/core/fetch/trust_token.idl.
    """

    # Description is missing from the devtools protocol document. # noqa
    operation: TrustTokenOperationType
    # Only set for "token-redemption" operation and determine whether to requesta fresh SRR or use a still valid cached SRR. # noqa
    refresh_policy: typing.List[typing.Literal["UseCached", "Refresh"]]
    # Origins of issuers from whom to request tokens or redemption records. # noqa
    issuers: typing.Optional[str]


class TrustTokenOperationType(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    ISSUANCE = "issuance"
    REDEMPTION = "redemption"
    SIGNING = "signing"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class AlternateProtocolUsage(str, enum.Enum):
    """The reason why Chrome uses a specific transport protocol for HTTP semantics."""

    ALTERNATIVE_JOB_WON_WITHOUT_RACE = "alternative_job_won_without_race"
    ALTERNATIVE_JOB_WON_RACE = "alternative_job_won_race"
    MAIN_JOB_WON_RACE = "main_job_won_race"
    MAPPING_MISSING = "mapping_missing"
    BROKEN = "broken"
    DNS_ALPN_H3_JOB_WON_WITHOUT_RACE = "dns_alpn_h3_job_won_without_race"
    DNS_ALPN_H3_JOB_WON_RACE = "dns_alpn_h3_job_won_race"
    UNSPECIFIED_REASON = "unspecified_reason"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class Response:
    """HTTP response data."""

    # Response URL. This URL can be different from CachedResource.url in case ofredirect. # noqa
    url: str
    # HTTP response status code. # noqa
    status: int
    # HTTP response status text. # noqa
    status_text: str
    # HTTP response headers. # noqa
    headers: Headers
    # Resource mimeType as determined by the browser. # noqa
    mime_type: str
    # Specifies whether physical connection was actually reused for thisrequest. # noqa
    connection_reused: bool
    # Physical connection id that was actually used for this request. # noqa
    connection_id: float
    # Total number of bytes received for this request so far. # noqa
    encoded_data_length: float
    # Security state of the request resource. # noqa
    security_state: security.SecurityState
    # HTTP response headers text. This has been replaced by the headers inNetwork.responseReceivedExtraInfo. # noqa
    headers_text: typing.Optional[str]
    # Refined HTTP request headers that were actually transmitted over thenetwork. # noqa
    request_headers: typing.Optional[Headers]
    # HTTP request headers text. This has been replaced by the headers inNetwork.requestWillBeSentExtraInfo. # noqa
    request_headers_text: typing.Optional[str]
    # Remote IP address. # noqa
    remote_ip_address: typing.Optional[str]
    # Remote port. # noqa
    remote_port: typing.Optional[int]
    # Specifies that the request was served from the disk cache. # noqa
    from_disk_cache: typing.Optional[bool]
    # Specifies that the request was served from the ServiceWorker. # noqa
    from_service_worker: typing.Optional[bool]
    # Specifies that the request was served from the prefetch cache. # noqa
    from_prefetch_cache: typing.Optional[bool]
    # Timing information for the given request. # noqa
    timing: typing.Optional[ResourceTiming]
    # Response source of response from ServiceWorker. # noqa
    service_worker_response_source: typing.Optional[ServiceWorkerResponseSource]
    # The time at which the returned response was generated. # noqa
    response_time: typing.Optional[TimeSinceEpoch]
    # Cache Storage Cache Name. # noqa
    cache_storage_cache_name: typing.Optional[str]
    # Protocol used to fetch this request. # noqa
    protocol: typing.Optional[str]
    # The reason why Chrome uses a specific transport protocol for HTTPsemantics. # noqa
    alternate_protocol_usage: typing.Optional[AlternateProtocolUsage]
    # Security details for the request. # noqa
    security_details: typing.Optional[SecurityDetails]


@dataclass
class WebSocketRequest:
    """WebSocket request data."""

    # HTTP request headers. # noqa
    headers: Headers


@dataclass
class WebSocketResponse:
    """WebSocket response data."""

    # HTTP response status code. # noqa
    status: int
    # HTTP response status text. # noqa
    status_text: str
    # HTTP response headers. # noqa
    headers: Headers
    # HTTP response headers text. # noqa
    headers_text: typing.Optional[str]
    # HTTP request headers. # noqa
    request_headers: typing.Optional[Headers]
    # HTTP request headers text. # noqa
    request_headers_text: typing.Optional[str]


@dataclass
class WebSocketFrame:
    """WebSocket message data.

    This represents an entire WebSocket message, not just a fragmented frame as the name suggests.
    """

    # WebSocket message opcode. # noqa
    opcode: float
    # WebSocket message mask. # noqa
    mask: bool
    # WebSocket message payload data. If the opcode is 1, this is a text messageand payloadData is a UTF-8 string. If the opcode isn't 1, then payloadData is abase64 encoded string representing binary data. # noqa
    payload_data: str


@dataclass
class CachedResource:
    """Information about the cached resource."""

    # Resource URL. This is the url of the original network request. # noqa
    url: str
    # Type of this resource. # noqa
    type: ResourceType
    # Cached response body size. # noqa
    body_size: float
    # Cached response data. # noqa
    response: typing.Optional[Response]


@dataclass
class Initiator:
    """Information about the request initiator."""

    # Type of this initiator. # noqa
    type: typing.List[typing.Literal["parser", "script", "preload", "SignedExchange", "preflight", "other"]]
    # Initiator JavaScript stack trace, set for Script only. # noqa
    stack: typing.Optional[runtime.StackTrace]
    # Initiator URL, set for Parser type or for Script type (when script isimporting module) or for SignedExchange type. # noqa
    url: typing.Optional[str]
    # Initiator line number, set for Parser type or for Script type (when scriptis importing module) (0-based). # noqa
    line_number: typing.Optional[float]
    # Initiator column number, set for Parser type or for Script type (whenscript is importing module) (0-based). # noqa
    column_number: typing.Optional[float]
    # Set if another request triggered this request (e.g. preflight). # noqa
    request_id: typing.Optional[RequestId]


@dataclass
class Cookie:
    """Cookie object."""

    # Cookie name. # noqa
    name: str
    # Cookie value. # noqa
    value: str
    # Cookie domain. # noqa
    domain: str
    # Cookie path. # noqa
    path: str
    # Cookie expiration date as the number of seconds since the UNIX epoch. # noqa
    expires: float
    # Cookie size. # noqa
    size: int
    # True if cookie is http-only. # noqa
    http_only: bool
    # True if cookie is secure. # noqa
    secure: bool
    # True in case of session cookie. # noqa
    session: bool
    # Cookie Priority # noqa
    priority: CookiePriority
    # True if cookie is SameParty. # noqa
    same_party: bool
    # Cookie source scheme type. # noqa
    source_scheme: CookieSourceScheme
    # Cookie source port. Valid values are {-1, [1, 65535]}, -1 indicates anunspecified port. An unspecified port value allows protocol clients to emulatelegacy cookie scope for the port. This is a temporary ability and it will beremoved in the future. # noqa
    source_port: int
    # Cookie SameSite type. # noqa
    same_site: typing.Optional[CookieSameSite]
    # Cookie partition key. The site of the top-level URL the browser wasvisiting at the start of the request to the endpoint that set the cookie. # noqa
    partition_key: typing.Optional[str]
    # True if cookie partition key is opaque. # noqa
    partition_key_opaque: typing.Optional[bool]


class SetCookieBlockedReason(str, enum.Enum):
    """Types of reasons why a cookie may not be stored from a response."""

    SECURE_ONLY = "secure_only"
    SAME_SITE_STRICT = "same_site_strict"
    SAME_SITE_LAX = "same_site_lax"
    SAME_SITE_UNSPECIFIED_TREATED_AS_LAX = "same_site_unspecified_treated_as_lax"
    SAME_SITE_NONE_INSECURE = "same_site_none_insecure"
    USER_PREFERENCES = "user_preferences"
    THIRD_PARTY_BLOCKED_IN_FIRST_PARTY_SET = "third_party_blocked_in_first_party_set"
    SYNTAX_ERROR = "syntax_error"
    SCHEME_NOT_SUPPORTED = "scheme_not_supported"
    OVERWRITE_SECURE = "overwrite_secure"
    INVALID_DOMAIN = "invalid_domain"
    INVALID_PREFIX = "invalid_prefix"
    UNKNOWN_ERROR = "unknown_error"
    SCHEMEFUL_SAME_SITE_STRICT = "schemeful_same_site_strict"
    SCHEMEFUL_SAME_SITE_LAX = "schemeful_same_site_lax"
    SCHEMEFUL_SAME_SITE_UNSPECIFIED_TREATED_AS_LAX = "schemeful_same_site_unspecified_treated_as_lax"
    SAME_PARTY_FROM_CROSS_PARTY_CONTEXT = "same_party_from_cross_party_context"
    SAME_PARTY_CONFLICTS_WITH_OTHER_ATTRIBUTES = "same_party_conflicts_with_other_attributes"
    NAME_VALUE_PAIR_EXCEEDS_MAX_SIZE = "name_value_pair_exceeds_max_size"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class CookieBlockedReason(str, enum.Enum):
    """Types of reasons why a cookie may not be sent with a request."""

    SECURE_ONLY = "secure_only"
    NOT_ON_PATH = "not_on_path"
    DOMAIN_MISMATCH = "domain_mismatch"
    SAME_SITE_STRICT = "same_site_strict"
    SAME_SITE_LAX = "same_site_lax"
    SAME_SITE_UNSPECIFIED_TREATED_AS_LAX = "same_site_unspecified_treated_as_lax"
    SAME_SITE_NONE_INSECURE = "same_site_none_insecure"
    USER_PREFERENCES = "user_preferences"
    THIRD_PARTY_BLOCKED_IN_FIRST_PARTY_SET = "third_party_blocked_in_first_party_set"
    UNKNOWN_ERROR = "unknown_error"
    SCHEMEFUL_SAME_SITE_STRICT = "schemeful_same_site_strict"
    SCHEMEFUL_SAME_SITE_LAX = "schemeful_same_site_lax"
    SCHEMEFUL_SAME_SITE_UNSPECIFIED_TREATED_AS_LAX = "schemeful_same_site_unspecified_treated_as_lax"
    SAME_PARTY_FROM_CROSS_PARTY_CONTEXT = "same_party_from_cross_party_context"
    NAME_VALUE_PAIR_EXCEEDS_MAX_SIZE = "name_value_pair_exceeds_max_size"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class BlockedSetCookieWithReason:
    """A cookie which was not stored from a response with the corresponding reason."""

    # The reason(s) this cookie was blocked. # noqa
    blocked_reasons: SetCookieBlockedReason
    # The string representing this individual cookie as it would appear in theheader. This is not the entire "cookie" or "set-cookie" header which could havemultiple cookies. # noqa
    cookie_line: str
    # The cookie object which represents the cookie which was not stored. It isoptional because sometimes complete cookie information is not available, such asin the case of parsing errors. # noqa
    cookie: typing.Optional[Cookie]


@dataclass
class BlockedCookieWithReason:
    """A cookie with was not sent with a request with the corresponding reason."""

    # The reason(s) the cookie was blocked. # noqa
    blocked_reasons: CookieBlockedReason
    # The cookie object representing the cookie which was not sent. # noqa
    cookie: Cookie


@dataclass
class CookieParam:
    """Cookie parameter object."""

    # Cookie name. # noqa
    name: str
    # Cookie value. # noqa
    value: str
    # The request-URI to associate with the setting of the cookie. This valuecan affect the default domain, path, source port, and source scheme values ofthe created cookie. # noqa
    url: typing.Optional[str]
    # Cookie domain. # noqa
    domain: typing.Optional[str]
    # Cookie path. # noqa
    path: typing.Optional[str]
    # True if cookie is secure. # noqa
    secure: typing.Optional[bool]
    # True if cookie is http-only. # noqa
    http_only: typing.Optional[bool]
    # Cookie SameSite type. # noqa
    same_site: typing.Optional[CookieSameSite]
    # Cookie expiration date, session cookie if not set # noqa
    expires: typing.Optional[TimeSinceEpoch]
    # Cookie Priority. # noqa
    priority: typing.Optional[CookiePriority]
    # True if cookie is SameParty. # noqa
    same_party: typing.Optional[bool]
    # Cookie source scheme type. # noqa
    source_scheme: typing.Optional[CookieSourceScheme]
    # Cookie source port. Valid values are {-1, [1, 65535]}, -1 indicates anunspecified port. An unspecified port value allows protocol clients to emulatelegacy cookie scope for the port. This is a temporary ability and it will beremoved in the future. # noqa
    source_port: typing.Optional[int]
    # Cookie partition key. The site of the top-level URL the browser wasvisiting at the start of the request to the endpoint that set the cookie. If notset, the cookie will be set as not partitioned. # noqa
    partition_key: typing.Optional[str]


@dataclass
class AuthChallenge:
    """Authorization challenge for HTTP status code 401 or 407."""

    # Origin of the challenger. # noqa
    origin: str
    # The authentication scheme used, such as basic or digest # noqa
    scheme: str
    # The realm of the challenge. May be empty. # noqa
    realm: str
    # Source of the authentication challenge. # noqa
    source: typing.Optional[typing.List[typing.Literal["Server", "Proxy"]]]


@dataclass
class AuthChallengeResponse:
    """Response to an AuthChallenge."""

    # The decision on what to do in response to the authorization challenge.Default means deferring to the default behavior of the net stack, which willlikely either the Cancel authentication or display a popup dialog box. # noqa
    response: typing.List[typing.Literal["Default", "CancelAuth", "ProvideCredentials"]]
    # The username to provide, possibly empty. Should only be set if response isProvideCredentials. # noqa
    username: typing.Optional[str]
    # The password to provide, possibly empty. Should only be set if response isProvideCredentials. # noqa
    password: typing.Optional[str]


class InterceptionStage(str, enum.Enum):
    """Stages of the interception to begin intercepting.

    Request will intercept before the request is sent. Response will intercept after the response is received.
    """

    REQUEST = "request"
    HEADERS_RECEIVED = "headers_received"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class RequestPattern:
    """Request pattern for interception."""

    # Wildcards (`'*'` -> zero or more, `'?'` -> exactly one) are allowed.Escape character is backslash. Omitting is equivalent to `"*"`. # noqa
    url_pattern: typing.Optional[str]
    # If set, only requests for matching resource types will be intercepted. # noqa
    resource_type: typing.Optional[ResourceType]
    # Stage at which to begin intercepting requests. Default is Request. # noqa
    interception_stage: typing.Optional[InterceptionStage]


@dataclass
class SignedExchangeSignature:
    """Information about a signed exchange signature.

    https://wicg.github.io/webpackage/draft-yasskin-httpbis-origin-signed-exchanges-impl.html#rfc.section.3.1
    """

    # Signed exchange signature label. # noqa
    label: str
    # The hex string of signed exchange signature. # noqa
    signature: str
    # Signed exchange signature integrity. # noqa
    integrity: str
    # Signed exchange signature validity Url. # noqa
    validity_url: str
    # Signed exchange signature date. # noqa
    date: int
    # Signed exchange signature expires. # noqa
    expires: int
    # Signed exchange signature cert Url. # noqa
    cert_url: typing.Optional[str]
    # The hex string of signed exchange signature cert sha256. # noqa
    cert_sha256: typing.Optional[str]
    # The encoded certificates. # noqa
    certificates: typing.Optional[str]


@dataclass
class SignedExchangeHeader:
    """Information about a signed exchange header.

    https://wicg.github.io/webpackage/draft-yasskin-httpbis-origin-signed-exchanges-impl.html#cbor-representation
    """

    # Signed exchange request URL. # noqa
    request_url: str
    # Signed exchange response code. # noqa
    response_code: int
    # Signed exchange response headers. # noqa
    response_headers: Headers
    # Signed exchange response signature. # noqa
    signatures: SignedExchangeSignature
    # Signed exchange header integrity hash in the form of "sha256-<base64-hash-value>". # noqa
    header_integrity: str


class SignedExchangeErrorField(str, enum.Enum):
    """Field type for a signed exchange related error."""

    SIGNATURE_SIG = "signature_sig"
    SIGNATURE_INTEGRITY = "signature_integrity"
    SIGNATURE_CERT_URL = "signature_cert_url"
    SIGNATURE_CERT_SHA256 = "signature_cert_sha256"
    SIGNATURE_VALIDITY_URL = "signature_validity_url"
    SIGNATURE_TIMESTAMPS = "signature_timestamps"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class SignedExchangeError:
    """Information about a signed exchange response."""

    # Error message. # noqa
    message: str
    # The index of the signature which caused the error. # noqa
    signature_index: typing.Optional[int]
    # The field which caused the error. # noqa
    error_field: typing.Optional[SignedExchangeErrorField]


@dataclass
class SignedExchangeInfo:
    """Information about a signed exchange response."""

    # The outer response of signed HTTP exchange which was received fromnetwork. # noqa
    outer_response: Response
    # Information about the signed exchange header. # noqa
    header: typing.Optional[SignedExchangeHeader]
    # Security details for the signed exchange header. # noqa
    security_details: typing.Optional[SecurityDetails]
    # Errors occurred while handling the signed exchagne. # noqa
    errors: typing.Optional[SignedExchangeError]


class ContentEncoding(str, enum.Enum):
    """List of content encodings supported by the backend."""

    DEFLATE = "deflate"
    GZIP = "gzip"
    BR = "br"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class PrivateNetworkRequestPolicy(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    ALLOW = "allow"
    BLOCK_FROM_INSECURE_TO_MORE_PRIVATE = "block_from_insecure_to_more_private"
    WARN_FROM_INSECURE_TO_MORE_PRIVATE = "warn_from_insecure_to_more_private"
    PREFLIGHT_BLOCK = "preflight_block"
    PREFLIGHT_WARN = "preflight_warn"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class IPAddressSpace(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    LOCAL = "local"
    PRIVATE = "private"
    PUBLIC = "public"
    UNKNOWN = "unknown"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class ConnectTiming:
    """Description is missing from the devtools protocol document."""

    # Timing's requestTime is a baseline in seconds, while the other numbers areticks in milliseconds relatively to this requestTime. Matches ResourceTiming'srequestTime for the same request (but not for redirected requests). # noqa
    request_time: float


@dataclass
class ClientSecurityState:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    initiator_is_secure_context: bool
    # Description is missing from the devtools protocol document. # noqa
    initiator_ip_address_space: IPAddressSpace
    # Description is missing from the devtools protocol document. # noqa
    private_network_request_policy: PrivateNetworkRequestPolicy


class CrossOriginOpenerPolicyValue(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    SAME_ORIGIN = "same_origin"
    SAME_ORIGIN_ALLOW_POPUPS = "same_origin_allow_popups"
    RESTRICT_PROPERTIES = "restrict_properties"
    UNSAFE_NONE = "unsafe_none"
    SAME_ORIGIN_PLUS_COEP = "same_origin_plus_coep"
    RESTRICT_PROPERTIES_PLUS_COEP = "restrict_properties_plus_coep"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class CrossOriginOpenerPolicyStatus:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    value: CrossOriginOpenerPolicyValue
    # Description is missing from the devtools protocol document. # noqa
    report_only_value: CrossOriginOpenerPolicyValue
    # Description is missing from the devtools protocol document. # noqa
    reporting_endpoint: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    report_only_reporting_endpoint: typing.Optional[str]


class CrossOriginEmbedderPolicyValue(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    NONE = "none"
    CREDENTIALLESS = "credentialless"
    REQUIRE_CORP = "require_corp"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class CrossOriginEmbedderPolicyStatus:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    value: CrossOriginEmbedderPolicyValue
    # Description is missing from the devtools protocol document. # noqa
    report_only_value: CrossOriginEmbedderPolicyValue
    # Description is missing from the devtools protocol document. # noqa
    reporting_endpoint: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    report_only_reporting_endpoint: typing.Optional[str]


@dataclass
class SecurityIsolationStatus:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    coop: typing.Optional[CrossOriginOpenerPolicyStatus]
    # Description is missing from the devtools protocol document. # noqa
    coep: typing.Optional[CrossOriginEmbedderPolicyStatus]


class ReportStatus(str, enum.Enum):
    """The status of a Reporting API report."""

    QUEUED = "queued"
    PENDING = "pending"
    MARKED_FOR_REMOVAL = "marked_for_removal"
    SUCCESS = "success"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class ReportId(str):
    """Description is missing from the devtools protocol document."""

    def to_json(self) -> ReportId:
        return self

    @classmethod
    def from_json(cls, value: str) -> ReportId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class ReportingApiReport:
    """An object representing a report generated by the Reporting API."""

    # Description is missing from the devtools protocol document. # noqa
    id: ReportId
    # The URL of the document that triggered the report. # noqa
    initiator_url: str
    # The name of the endpoint group that should be used to deliver the report. # noqa
    destination: str
    # The type of the report (specifies the set of data that is contained in thereport body). # noqa
    type: str
    # When the report was generated. # noqa
    timestamp: TimeSinceEpoch
    # How many uploads deep the related request was. # noqa
    depth: int
    # The number of delivery attempts made so far, not including an activeattempt. # noqa
    completed_attempts: int
    # Description is missing from the devtools protocol document. # noqa
    body: typing.Any
    # Description is missing from the devtools protocol document. # noqa
    status: ReportStatus


@dataclass
class ReportingApiEndpoint:
    """Description is missing from the devtools protocol document."""

    # The URL of the endpoint to which reports may be delivered. # noqa
    url: str
    # Name of the endpoint group. # noqa
    group_name: str


@dataclass
class LoadNetworkResourcePageResult:
    """An object providing the result of a network resource load."""

    # Description is missing from the devtools protocol document. # noqa
    success: bool
    # Optional values used for error reporting. # noqa
    net_error: typing.Optional[float]
    # Description is missing from the devtools protocol document. # noqa
    net_error_name: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    http_status_code: typing.Optional[float]
    # If successful, one of the following two fields holds the result. # noqa
    stream: typing.Optional[io.StreamHandle]
    # Response headers. # noqa
    headers: typing.Optional[Headers]


@dataclass
class LoadNetworkResourceOptions:
    """An options object that may be extended later to better support CORS, CORB and streaming."""

    # Description is missing from the devtools protocol document. # noqa
    disable_cache: bool
    # Description is missing from the devtools protocol document. # noqa
    include_credentials: bool


@dataclass
@memoize_event("Network.dataReceived")
class DataReceived:
    """Fired when data chunk was received over the network."""

    request_id: RequestId
    timestamp: MonotonicTime
    data_length: int
    encoded_data_length: int


@dataclass
@memoize_event("Network.eventSourceMessageReceived")
class EventSourceMessageReceived:
    """Fired when EventSource message is received."""

    request_id: RequestId
    timestamp: MonotonicTime
    event_name: str
    event_id: str
    data: str


@dataclass
@memoize_event("Network.loadingFailed")
class LoadingFailed:
    """Fired when HTTP request has failed to load."""

    request_id: RequestId
    timestamp: MonotonicTime
    type: ResourceType
    error_text: str
    canceled: typing.Optional[bool]
    blocked_reason: typing.Optional[BlockedReason]
    cors_error_status: typing.Optional[CorsErrorStatus]


@dataclass
@memoize_event("Network.loadingFinished")
class LoadingFinished:
    """Fired when HTTP request has finished loading."""

    request_id: RequestId
    timestamp: MonotonicTime
    encoded_data_length: float
    should_report_corb_blocking: typing.Optional[bool]


@dataclass
@memoize_event("Network.requestIntercepted")
class RequestIntercepted:
    """Details of an intercepted HTTP request, which must be either allowed, blocked, modified or mocked.

    Deprecated, use Fetch.requestPaused instead.
    """

    interception_id: InterceptionId
    request: Request
    frame_id: page.FrameId
    resource_type: ResourceType
    is_navigation_request: bool
    is_download: typing.Optional[bool]
    redirect_url: typing.Optional[str]
    auth_challenge: typing.Optional[AuthChallenge]
    response_error_reason: typing.Optional[ErrorReason]
    response_status_code: typing.Optional[int]
    response_headers: typing.Optional[Headers]
    request_id: typing.Optional[RequestId]


@dataclass
@memoize_event("Network.requestServedFromCache")
class RequestServedFromCache:
    """Fired if request ended up loading from cache."""

    request_id: RequestId


@dataclass
@memoize_event("Network.requestWillBeSent")
class RequestWillBeSent:
    """Fired when page is about to send HTTP request."""

    request_id: RequestId
    loader_id: LoaderId
    document_url: str
    request: Request
    timestamp: MonotonicTime
    wall_time: TimeSinceEpoch
    initiator: Initiator
    redirect_has_extra_info: bool
    redirect_response: typing.Optional[Response]
    type: typing.Optional[ResourceType]
    frame_id: typing.Optional[page.FrameId]
    has_user_gesture: typing.Optional[bool]


@dataclass
@memoize_event("Network.resourceChangedPriority")
class ResourceChangedPriority:
    """Fired when resource loading priority is changed."""

    request_id: RequestId
    new_priority: ResourcePriority
    timestamp: MonotonicTime


@dataclass
@memoize_event("Network.signedExchangeReceived")
class SignedExchangeReceived:
    """Fired when a signed exchange was received over the network."""

    request_id: RequestId
    info: SignedExchangeInfo


@dataclass
@memoize_event("Network.responseReceived")
class ResponseReceived:
    """Fired when HTTP response is available."""

    request_id: RequestId
    loader_id: LoaderId
    timestamp: MonotonicTime
    type: ResourceType
    response: Response
    has_extra_info: bool
    frame_id: typing.Optional[page.FrameId]


@dataclass
@memoize_event("Network.webSocketClosed")
class WebSocketClosed:
    """Fired when WebSocket is closed."""

    request_id: RequestId
    timestamp: MonotonicTime


@dataclass
@memoize_event("Network.webSocketCreated")
class WebSocketCreated:
    """Fired upon WebSocket creation."""

    request_id: RequestId
    url: str
    initiator: typing.Optional[Initiator]


@dataclass
@memoize_event("Network.webSocketFrameError")
class WebSocketFrameError:
    """Fired when WebSocket message error occurs."""

    request_id: RequestId
    timestamp: MonotonicTime
    error_message: str


@dataclass
@memoize_event("Network.webSocketFrameReceived")
class WebSocketFrameReceived:
    """Fired when WebSocket message is received."""

    request_id: RequestId
    timestamp: MonotonicTime
    response: WebSocketFrame


@dataclass
@memoize_event("Network.webSocketFrameSent")
class WebSocketFrameSent:
    """Fired when WebSocket message is sent."""

    request_id: RequestId
    timestamp: MonotonicTime
    response: WebSocketFrame


@dataclass
@memoize_event("Network.webSocketHandshakeResponseReceived")
class WebSocketHandshakeResponseReceived:
    """Fired when WebSocket handshake response becomes available."""

    request_id: RequestId
    timestamp: MonotonicTime
    response: WebSocketResponse


@dataclass
@memoize_event("Network.webSocketWillSendHandshakeRequest")
class WebSocketWillSendHandshakeRequest:
    """Fired when WebSocket is about to initiate handshake."""

    request_id: RequestId
    timestamp: MonotonicTime
    wall_time: TimeSinceEpoch
    request: WebSocketRequest


@dataclass
@memoize_event("Network.webTransportCreated")
class WebTransportCreated:
    """Fired upon WebTransport creation."""

    transport_id: RequestId
    url: str
    timestamp: MonotonicTime
    initiator: typing.Optional[Initiator]


@dataclass
@memoize_event("Network.webTransportConnectionEstablished")
class WebTransportConnectionEstablished:
    """Fired when WebTransport handshake is finished."""

    transport_id: RequestId
    timestamp: MonotonicTime


@dataclass
@memoize_event("Network.webTransportClosed")
class WebTransportClosed:
    """Fired when WebTransport is disposed."""

    transport_id: RequestId
    timestamp: MonotonicTime


@dataclass
@memoize_event("Network.requestWillBeSentExtraInfo")
class RequestWillBeSentExtraInfo:
    """Fired when additional information about a requestWillBeSent event is available from the network stack.

    Not every requestWillBeSent event will have an additional requestWillBeSentExtraInfo fired for it, and there is no
    guarantee whether requestWillBeSent or requestWillBeSentExtraInfo will be fired first for the same request.
    """

    request_id: RequestId
    associated_cookies: typing.List[BlockedCookieWithReason]
    headers: Headers
    connect_timing: ConnectTiming
    client_security_state: typing.Optional[ClientSecurityState]
    site_has_cookie_in_other_partition: typing.Optional[bool]


@dataclass
@memoize_event("Network.responseReceivedExtraInfo")
class ResponseReceivedExtraInfo:
    """Fired when additional information about a responseReceived event is available from the network stack.

    Not every responseReceived event will have an additional responseReceivedExtraInfo for it, and
    responseReceivedExtraInfo may be fired before or after responseReceived.
    """

    request_id: RequestId
    blocked_cookies: typing.List[BlockedSetCookieWithReason]
    headers: Headers
    resource_ip_address_space: IPAddressSpace
    status_code: int
    headers_text: typing.Optional[str]
    cookie_partition_key: typing.Optional[str]
    cookie_partition_key_opaque: typing.Optional[bool]


@dataclass
@memoize_event("Network.trustTokenOperationDone")
class TrustTokenOperationDone:
    """Fired exactly once for each Trust Token operation.

    Depending on the type of the operation and whether the operation succeeded or failed, the event is fired before the
    corresponding request was sent or after the response was received.
    """

    status: typing.Literal[
        "ok",
        "invalid_argument",
        "failed_precondition",
        "resource_exhausted",
        "already_exists",
        "unavailable",
        "unauthorized",
        "bad_response",
        "internal_error",
        "unknown_error",
        "fulfilled_locally",
    ]
    type: TrustTokenOperationType
    request_id: RequestId
    top_level_origin: typing.Optional[str]
    issuer_origin: typing.Optional[str]
    issued_token_count: typing.Optional[int]


@dataclass
@memoize_event("Network.subresourceWebBundleMetadataReceived")
class SubresourceWebBundleMetadataReceived:
    """Fired once when parsing the .wbn file has succeeded.

    The event contains the information about the web bundle contents.
    """

    request_id: RequestId
    urls: typing.List[str]


@dataclass
@memoize_event("Network.subresourceWebBundleMetadataError")
class SubresourceWebBundleMetadataError:
    """Fired once when parsing the .wbn file has failed."""

    request_id: RequestId
    error_message: str


@dataclass
@memoize_event("Network.subresourceWebBundleInnerResponseParsed")
class SubresourceWebBundleInnerResponseParsed:
    """Fired when handling requests for resources within a .wbn file.

    Note: this will only be fired for resources that are requested by the webpage.
    """

    inner_request_id: RequestId
    inner_request_url: str
    bundle_request_id: typing.Optional[RequestId]


@dataclass
@memoize_event("Network.subresourceWebBundleInnerResponseError")
class SubresourceWebBundleInnerResponseError:
    """Fired when request for resources within a .wbn file failed."""

    inner_request_id: RequestId
    inner_request_url: str
    error_message: str
    bundle_request_id: typing.Optional[RequestId]


@dataclass
@memoize_event("Network.reportingApiReportAdded")
class ReportingApiReportAdded:
    """Is sent whenever a new report is added.

    And after 'enableReportingApi' for all existing reports.
    """

    report: ReportingApiReport


@dataclass
@memoize_event("Network.reportingApiReportUpdated")
class ReportingApiReportUpdated:
    """Description is missing from the devtools protocol document."""

    report: ReportingApiReport


@dataclass
@memoize_event("Network.reportingApiEndpointsChangedForOrigin")
class ReportingApiEndpointsChangedForOrigin:
    """Description is missing from the devtools protocol document."""

    origin: str
    endpoints: typing.List[ReportingApiEndpoint]


async def set_accepted_encodings() -> None:
    """Sets a list of content encodings that will be accepted.

    Empty list means no encoding is accepted. # noqa
    """
    ...


async def clear_accepted_encodings_override() -> None:
    """Clears accepted encodings set by setAcceptedEncodings # noqa."""
    ...


async def can_clear_browser_cache() -> None:
    """Tells whether clearing browser cache is supported.

    # noqa
    """
    ...


async def can_clear_browser_cookies() -> None:
    """Tells whether clearing browser cookies is supported.

    # noqa
    """
    ...


async def can_emulate_network_conditions() -> None:
    """Tells whether emulation of network conditions is supported.

    # noqa
    """
    ...


async def clear_browser_cache() -> None:
    """Clears browser cache.

    # noqa
    """
    ...


async def clear_browser_cookies() -> None:
    """Clears browser cookies.

    # noqa
    """
    ...


async def continue_intercepted_request() -> None:
    """Response to Network.requestIntercepted which either modifies the request to continue with any modifications, or
    blocks it, or completes it with the provided response bytes.

    If a network fetch occurs as a result which encounters a redirect an additional Network.requestIntercepted event
    will be sent with the same InterceptionId. Deprecated, use Fetch.continueRequest, Fetch.fulfillRequest and
    Fetch.failRequest instead. # noqa
    """
    ...


async def delete_cookies() -> None:
    """Deletes browser cookies with matching name and url or domain/path pair.

    # noqa
    """
    ...


async def disable() -> None:
    """Disables network tracking, prevents network events from being sent to the client.

    # noqa
    """
    ...


async def emulate_network_conditions() -> None:
    """Activates emulation of network conditions.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables network tracking, network events will now be delivered to the client.

    # noqa
    """
    ...


async def get_all_cookies() -> None:
    """Returns all browser cookies.

    Depending on the backend support, will return detailed cookie information in the `cookies` field. Deprecated. Use
    Storage.getCookies instead. # noqa
    """
    ...


async def get_certificate() -> None:
    """Returns the DER-encoded certificate.

    # noqa
    """
    ...


async def get_cookies() -> None:
    """Returns all browser cookies for the current URL.

    Depending on the backend support, will return detailed cookie information in the `cookies` field. # noqa
    """
    ...


async def get_response_body() -> None:
    """Returns content served for the given request.

    # noqa
    """
    ...


async def get_request_post_data() -> None:
    """Returns post data sent with the request.

    Returns an error when no data was sent with the request. # noqa
    """
    ...


async def get_response_body_for_interception() -> None:
    """Returns content served for the given currently intercepted request.

    # noqa
    """
    ...


async def take_response_body_for_interception_as_stream() -> None:
    """Returns a handle to the stream representing the response body.

    Note that after this command, the intercepted request can't be continued as is -- you either need to cancel it or to
    provide the response body. The stream only supports sequential read, IO.read will fail if the position is specified.
    # noqa
    """
    ...


async def replay_xhr() -> None:
    """This method sends a new XMLHttpRequest which is identical to the original one.

    The following parameters should be identical: method, url, async, request body, extra headers, withCredentials
    attribute, user, password. # noqa
    """
    ...


async def search_in_response_body() -> None:
    """Searches for given string in response content.

    # noqa
    """
    ...


async def set_blocked_ur_ls() -> None:
    """Blocks URLs from loading.

    # noqa
    """
    ...


async def set_bypass_service_worker() -> None:
    """Toggles ignoring of service worker for each request.

    # noqa
    """
    ...


async def set_cache_disabled() -> None:
    """Toggles ignoring cache for each request.

    If `true`, cache will not be used. # noqa
    """
    ...


async def set_cookie() -> None:
    """Sets a cookie with the given cookie data; may overwrite equivalent cookies if they exist.

    # noqa
    """
    ...


async def set_cookies() -> None:
    """Sets given cookies.

    # noqa
    """
    ...


async def set_extra_http_headers() -> None:
    """Specifies whether to always send extra HTTP headers with the requests from this page.

    # noqa
    """
    ...


async def set_attach_debug_stack() -> None:
    """Specifies whether to attach a page script stack id in requests # noqa."""
    ...


async def set_request_interception() -> None:
    """Sets the requests to intercept that match the provided patterns and optionally resource types.

    Deprecated, please use Fetch.enable instead. # noqa
    """
    ...


async def set_user_agent_override() -> None:
    """Allows overriding user agent with the given string.

    # noqa
    """
    ...


async def get_security_isolation_status() -> None:
    """Returns information about the COEP/COOP isolation status.

    # noqa
    """
    ...


async def enable_reporting_api() -> None:
    """Enables tracking for the Reporting API, events generated by the Reporting API will now be delivered to the
    client.

    Enabling triggers 'reportingApiReportAdded' for all existing reports. # noqa
    """
    ...


async def load_network_resource() -> None:
    """Fetches the resource and returns the content.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Page`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Page/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from . import dom
from . import network
from . import runtime
from .utils import memoize_event


class FrameId(str):
    """Unique frame identifier."""

    def to_json(self) -> FrameId:
        return self

    @classmethod
    def from_json(cls, value: str) -> FrameId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class AdFrameType(str, enum.Enum):
    """Indicates whether a frame has been identified as an ad."""

    NONE = "none"
    CHILD = "child"
    ROOT = "root"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class AdFrameExplanation(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    PARENT_IS_AD = "parent_is_ad"
    CREATED_BY_AD_SCRIPT = "created_by_ad_script"
    MATCHED_BLOCKING_RULE = "matched_blocking_rule"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class AdFrameStatus:
    """Indicates whether a frame has been identified as an ad and why."""

    # Description is missing from the devtools protocol document. # noqa
    ad_frame_type: AdFrameType
    # Description is missing from the devtools protocol document. # noqa
    explanations: typing.Optional[AdFrameExplanation]


@dataclass
class AdScriptId:
    """Identifies the bottom-most script which caused the frame to be labelled as an ad."""

    # Script Id of the bottom-most script which caused the frame to be labelledas an ad. # noqa
    script_id: runtime.ScriptId
    # Id of adScriptId's debugger. # noqa
    debugger_id: runtime.UniqueDebuggerId


class SecureContextType(str, enum.Enum):
    """Indicates whether the frame is a secure context and why it is the case."""

    SECURE = "secure"
    SECURE_LOCALHOST = "secure_localhost"
    INSECURE_SCHEME = "insecure_scheme"
    INSECURE_ANCESTOR = "insecure_ancestor"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class CrossOriginIsolatedContextType(str, enum.Enum):
    """Indicates whether the frame is cross-origin isolated and why it is the case."""

    ISOLATED = "isolated"
    NOT_ISOLATED = "not_isolated"
    NOT_ISOLATED_FEATURE_DISABLED = "not_isolated_feature_disabled"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class GatedAPIFeatures(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    SHARED_ARRAY_BUFFERS = "shared_array_buffers"
    SHARED_ARRAY_BUFFERS_TRANSFER_ALLOWED = "shared_array_buffers_transfer_allowed"
    PERFORMANCE_MEASURE_MEMORY = "performance_measure_memory"
    PERFORMANCE_PROFILE = "performance_profile"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class PermissionsPolicyFeature(str, enum.Enum):
    """All Permissions Policy features.

    This enum should match the one defined
    in third_party/blink/renderer/core/permissions_policy/permissions_policy_features.json5.
    """

    ACCELEROMETER = "accelerometer"
    AMBIENT_LIGHT_SENSOR = "ambient-light-sensor"
    ATTRIBUTION_REPORTING = "attribution-reporting"
    AUTOPLAY = "autoplay"
    BLUETOOTH = "bluetooth"
    BROWSING_TOPICS = "browsing-topics"
    CAMERA = "camera"
    CH_DPR = "ch-dpr"
    CH_DEVICE_MEMORY = "ch-device-memory"
    CH_DOWNLINK = "ch-downlink"
    CH_ECT = "ch-ect"
    CH_PREFERS_COLOR_SCHEME = "ch-prefers-color-scheme"
    CH_PREFERS_REDUCED_MOTION = "ch-prefers-reduced-motion"
    CH_RTT = "ch-rtt"
    CH_SAVE_DATA = "ch-save-data"
    CH_UA = "ch-ua"
    CH_UA_ARCH = "ch-ua-arch"
    CH_UA_BITNESS = "ch-ua-bitness"
    CH_UA_PLATFORM = "ch-ua-platform"
    CH_UA_MODEL = "ch-ua-model"
    CH_UA_MOBILE = "ch-ua-mobile"
    CH_UA_FULL = "ch-ua-full"
    CH_UA_FULL_VERSION = "ch-ua-full-version"
    CH_UA_FULL_VERSION_LIST = "ch-ua-full-version-list"
    CH_UA_PLATFORM_VERSION = "ch-ua-platform-version"
    CH_UA_REDUCED = "ch-ua-reduced"
    CH_UA_WOW64 = "ch-ua-wow64"
    CH_VIEWPORT_HEIGHT = "ch-viewport-height"
    CH_VIEWPORT_WIDTH = "ch-viewport-width"
    CH_WIDTH = "ch-width"
    CLIPBOARD_READ = "clipboard-read"
    CLIPBOARD_WRITE = "clipboard-write"
    COMPUTE_PRESSURE = "compute-pressure"
    CROSS_ORIGIN_ISOLATED = "cross-origin-isolated"
    DIRECT_SOCKETS = "direct-sockets"
    DISPLAY_CAPTURE = "display-capture"
    DOCUMENT_DOMAIN = "document-domain"
    ENCRYPTED_MEDIA = "encrypted-media"
    EXECUTION_WHILE_OUT_OF_VIEWPORT = "execution-while-out-of-viewport"
    EXECUTION_WHILE_NOT_RENDERED = "execution-while-not-rendered"
    FOCUS_WITHOUT_USER_ACTIVATION = "focus-without-user-activation"
    FULLSCREEN = "fullscreen"
    FROBULATE = "frobulate"
    GAMEPAD = "gamepad"
    GEOLOCATION = "geolocation"
    GYROSCOPE = "gyroscope"
    HID = "hid"
    IDENTITY_CREDENTIALS_GET = "identity-credentials-get"
    IDLE_DETECTION = "idle-detection"
    INTEREST_COHORT = "interest-cohort"
    JOIN_AD_INTEREST_GROUP = "join-ad-interest-group"
    KEYBOARD_MAP = "keyboard-map"
    LOCAL_FONTS = "local-fonts"
    MAGNETOMETER = "magnetometer"
    MICROPHONE = "microphone"
    MIDI = "midi"
    OTP_CREDENTIALS = "otp-credentials"
    PAYMENT = "payment"
    PICTURE_IN_PICTURE = "picture-in-picture"
    PRIVATE_AGGREGATION = "private-aggregation"
    PUBLICKEY_CREDENTIALS_GET = "publickey-credentials-get"
    RUN_AD_AUCTION = "run-ad-auction"
    SCREEN_WAKE_LOCK = "screen-wake-lock"
    SERIAL = "serial"
    SHARED_AUTOFILL = "shared-autofill"
    SHARED_STORAGE = "shared-storage"
    SHARED_STORAGE_SELECT_URL = "shared-storage-select-url"
    SMART_CARD = "smart-card"
    STORAGE_ACCESS = "storage-access"
    SYNC_XHR = "sync-xhr"
    TRUST_TOKEN_REDEMPTION = "trust-token-redemption"
    UNLOAD = "unload"
    USB = "usb"
    VERTICAL_SCROLL = "vertical-scroll"
    WEB_SHARE = "web-share"
    WINDOW_MANAGEMENT = "window-management"
    WINDOW_PLACEMENT = "window-placement"
    XR_SPATIAL_TRACKING = "xr-spatial-tracking"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class PermissionsPolicyBlockReason(str, enum.Enum):
    """Reason for a permissions policy feature to be disabled."""

    HEADER = "header"
    IFRAME_ATTRIBUTE = "iframe_attribute"
    IN_FENCED_FRAME_TREE = "in_fenced_frame_tree"
    IN_ISOLATED_APP = "in_isolated_app"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class PermissionsPolicyBlockLocator:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    frame_id: FrameId
    # Description is missing from the devtools protocol document. # noqa
    block_reason: PermissionsPolicyBlockReason


@dataclass
class PermissionsPolicyFeatureState:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    feature: PermissionsPolicyFeature
    # Description is missing from the devtools protocol document. # noqa
    allowed: bool
    # Description is missing from the devtools protocol document. # noqa
    locator: typing.Optional[PermissionsPolicyBlockLocator]


class OriginTrialTokenStatus(str, enum.Enum):
    """Origin Trial(https://www.chromium.org/blink/origin-trials) support.

    Status for an Origin Trial token.
    """

    SUCCESS = "success"
    NOT_SUPPORTED = "not_supported"
    INSECURE = "insecure"
    EXPIRED = "expired"
    WRONG_ORIGIN = "wrong_origin"
    INVALID_SIGNATURE = "invalid_signature"
    MALFORMED = "malformed"
    WRONG_VERSION = "wrong_version"
    FEATURE_DISABLED = "feature_disabled"
    TOKEN_DISABLED = "token_disabled"
    FEATURE_DISABLED_FOR_USER = "feature_disabled_for_user"
    UNKNOWN_TRIAL = "unknown_trial"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class OriginTrialStatus(str, enum.Enum):
    """Status for an Origin Trial."""

    ENABLED = "enabled"
    VALID_TOKEN_NOT_PROVIDED = "valid_token_not_provided"
    O_S_NOT_SUPPORTED = "os_not_supported"
    TRIAL_NOT_ALLOWED = "trial_not_allowed"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class OriginTrialUsageRestriction(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    NONE = "none"
    SUBSET = "subset"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class OriginTrialToken:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    origin: str
    # Description is missing from the devtools protocol document. # noqa
    match_sub_domains: bool
    # Description is missing from the devtools protocol document. # noqa
    trial_name: str
    # Description is missing from the devtools protocol document. # noqa
    expiry_time: network.TimeSinceEpoch
    # Description is missing from the devtools protocol document. # noqa
    is_third_party: bool
    # Description is missing from the devtools protocol document. # noqa
    usage_restriction: OriginTrialUsageRestriction


@dataclass
class OriginTrialTokenWithStatus:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    raw_token_text: str
    # Description is missing from the devtools protocol document. # noqa
    status: OriginTrialTokenStatus
    # `parsedToken` is present only when the token is extractable and parsable. # noqa
    parsed_token: typing.Optional[OriginTrialToken]


@dataclass
class OriginTrial:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    trial_name: str
    # Description is missing from the devtools protocol document. # noqa
    status: OriginTrialStatus
    # Description is missing from the devtools protocol document. # noqa
    tokens_with_status: OriginTrialTokenWithStatus


@dataclass
class Frame:
    """Information about the Frame on the page."""

    # Frame unique identifier. # noqa
    id: FrameId
    # Identifier of the loader associated with this frame. # noqa
    loader_id: network.LoaderId
    # Frame document's URL without fragment. # noqa
    url: str
    # Frame document's registered domain, taking the public suffixes list intoaccount. Extracted from the Frame's url. Example URLs:http://www.google.com/file.html -> "google.com"http://a.b.co.uk/file.html      -> "b.co.uk" # noqa
    domain_and_registry: str
    # Frame document's security origin. # noqa
    security_origin: str
    # Frame document's mimeType as determined by the browser. # noqa
    mime_type: str
    # Indicates whether the main document is a secure context and explains whythat is the case. # noqa
    secure_context_type: SecureContextType
    # Indicates whether this is a cross origin isolated context. # noqa
    cross_origin_isolated_context_type: CrossOriginIsolatedContextType
    # Indicated which gated APIs / features are available. # noqa
    gated_api_features: GatedAPIFeatures
    # Parent frame identifier. # noqa
    parent_id: typing.Optional[FrameId]
    # Frame's name as specified in the tag. # noqa
    name: typing.Optional[str]
    # Frame document's URL fragment including the '#'. # noqa
    url_fragment: typing.Optional[str]
    # If the frame failed to load, this contains the URL that could not beloaded. Note that unlike url above, this URL may contain a fragment. # noqa
    unreachable_url: typing.Optional[str]
    # Indicates whether this frame was tagged as an ad and why. # noqa
    ad_frame_status: typing.Optional[AdFrameStatus]


@dataclass
class FrameResource:
    """Information about the Resource on the page."""

    # Resource URL. # noqa
    url: str
    # Type of this resource. # noqa
    type: network.ResourceType
    # Resource mimeType as determined by the browser. # noqa
    mime_type: str
    # last-modified timestamp as reported by server. # noqa
    last_modified: typing.Optional[network.TimeSinceEpoch]
    # Resource content size. # noqa
    content_size: typing.Optional[float]
    # True if the resource failed to load. # noqa
    failed: typing.Optional[bool]
    # True if the resource was canceled during loading. # noqa
    canceled: typing.Optional[bool]


@dataclass
class FrameResourceTree:
    """Information about the Frame hierarchy along with their cached resources."""

    # Frame information for this tree item. # noqa
    frame: Frame
    # Information about frame resources. # noqa
    resources: FrameResource
    # Child frames. # noqa
    child_frames: typing.Optional[FrameResourceTree]


@dataclass
class FrameTree:
    """Information about the Frame hierarchy."""

    # Frame information for this tree item. # noqa
    frame: Frame
    # Child frames. # noqa
    child_frames: typing.Optional[FrameTree]


class ScriptIdentifier(str):
    """Unique script identifier."""

    def to_json(self) -> ScriptIdentifier:
        return self

    @classmethod
    def from_json(cls, value: str) -> ScriptIdentifier:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class TransitionType(str, enum.Enum):
    """Transition type."""

    LINK = "link"
    TYPED = "typed"
    ADDRESS_BAR = "address_bar"
    AUTO_BOOKMARK = "auto_bookmark"
    AUTO_SUBFRAME = "auto_subframe"
    MANUAL_SUBFRAME = "manual_subframe"
    GENERATED = "generated"
    AUTO_TOPLEVEL = "auto_toplevel"
    FORM_SUBMIT = "form_submit"
    RELOAD = "reload"
    KEYWORD = "keyword"
    KEYWORD_GENERATED = "keyword_generated"
    OTHER = "other"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class NavigationEntry:
    """Navigation history entry."""

    # Unique id of the navigation history entry. # noqa
    id: int
    # URL of the navigation history entry. # noqa
    url: str
    # URL that the user typed in the url bar. # noqa
    user_typed_url: str
    # Title of the navigation history entry. # noqa
    title: str
    # Transition type. # noqa
    transition_type: TransitionType


@dataclass
class ScreencastFrameMetadata:
    """Screencast frame metadata."""

    # Top offset in DIP. # noqa
    offset_top: float
    # Page scale factor. # noqa
    page_scale_factor: float
    # Device screen width in DIP. # noqa
    device_width: float
    # Device screen height in DIP. # noqa
    device_height: float
    # Position of horizontal scroll in CSS pixels. # noqa
    scroll_offset_x: float
    # Position of vertical scroll in CSS pixels. # noqa
    scroll_offset_y: float
    # Frame swap timestamp. # noqa
    timestamp: typing.Optional[network.TimeSinceEpoch]


class DialogType(str, enum.Enum):
    """Javascript dialog type."""

    ALERT = "alert"
    CONFIRM = "confirm"
    PROMPT = "prompt"
    BEFOREUNLOAD = "beforeunload"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class AppManifestError:
    """Error while paring app manifest."""

    # Error message. # noqa
    message: str
    # If criticial, this is a non-recoverable parse error. # noqa
    critical: int
    # Error line. # noqa
    line: int
    # Error column. # noqa
    column: int


@dataclass
class AppManifestParsedProperties:
    """Parsed app manifest properties."""

    # Computed scope value # noqa
    scope: str


@dataclass
class LayoutViewport:
    """Layout viewport position and dimensions."""

    # Horizontal offset relative to the document (CSS pixels). # noqa
    page_x: int
    # Vertical offset relative to the document (CSS pixels). # noqa
    page_y: int
    # Width (CSS pixels), excludes scrollbar if present. # noqa
    client_width: int
    # Height (CSS pixels), excludes scrollbar if present. # noqa
    client_height: int


@dataclass
class VisualViewport:
    """Visual viewport position, dimensions, and scale."""

    # Horizontal offset relative to the layout viewport (CSS pixels). # noqa
    offset_x: float
    # Vertical offset relative to the layout viewport (CSS pixels). # noqa
    offset_y: float
    # Horizontal offset relative to the document (CSS pixels). # noqa
    page_x: float
    # Vertical offset relative to the document (CSS pixels). # noqa
    page_y: float
    # Width (CSS pixels), excludes scrollbar if present. # noqa
    client_width: float
    # Height (CSS pixels), excludes scrollbar if present. # noqa
    client_height: float
    # Scale relative to the ideal viewport (size at width=device-width). # noqa
    scale: float
    # Page zoom factor (CSS to device independent pixels ratio). # noqa
    zoom: typing.Optional[float]


@dataclass
class Viewport:
    """Viewport for capturing screenshot."""

    # X offset in device independent pixels (dip). # noqa
    x: float
    # Y offset in device independent pixels (dip). # noqa
    y: float
    # Rectangle width in device independent pixels (dip). # noqa
    width: float
    # Rectangle height in device independent pixels (dip). # noqa
    height: float
    # Page scale factor. # noqa
    scale: float


@dataclass
class FontFamilies:
    """Generic font families collection."""

    # The standard font-family. # noqa
    standard: typing.Optional[str]
    # The fixed font-family. # noqa
    fixed: typing.Optional[str]
    # The serif font-family. # noqa
    serif: typing.Optional[str]
    # The sansSerif font-family. # noqa
    sans_serif: typing.Optional[str]
    # The cursive font-family. # noqa
    cursive: typing.Optional[str]
    # The fantasy font-family. # noqa
    fantasy: typing.Optional[str]
    # The math font-family. # noqa
    math: typing.Optional[str]


@dataclass
class ScriptFontFamilies:
    """Font families collection for a script."""

    # Name of the script which these font families are defined for. # noqa
    script: str
    # Generic font families collection for the script. # noqa
    font_families: FontFamilies


@dataclass
class FontSizes:
    """Default font sizes."""

    # Default standard font size. # noqa
    standard: typing.Optional[int]
    # Default fixed font size. # noqa
    fixed: typing.Optional[int]


class ClientNavigationReason(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    FORM_SUBMISSION_GET = "form_submission_get"
    FORM_SUBMISSION_POST = "form_submission_post"
    HTTP_HEADER_REFRESH = "http_header_refresh"
    SCRIPT_INITIATED = "script_initiated"
    META_TAG_REFRESH = "meta_tag_refresh"
    PAGE_BLOCK_INTERSTITIAL = "page_block_interstitial"
    RELOAD = "reload"
    ANCHOR_CLICK = "anchor_click"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class ClientNavigationDisposition(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    CURRENT_TAB = "current_tab"
    NEW_TAB = "new_tab"
    NEW_WINDOW = "new_window"
    DOWNLOAD = "download"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class InstallabilityErrorArgument:
    """Description is missing from the devtools protocol document."""

    # Argument name (e.g. name:'minimum-icon-size-in-pixels'). # noqa
    name: str
    # Argument value (e.g. value:'64'). # noqa
    value: str


@dataclass
class InstallabilityError:
    """The installability error."""

    # The error id (e.g. 'manifest-missing-suitable-icon'). # noqa
    error_id: str
    # The list of error arguments (e.g. {name:'minimum-icon-size-in-pixels',value:'64'}). # noqa
    error_arguments: InstallabilityErrorArgument


class ReferrerPolicy(str, enum.Enum):
    """The referring-policy used for the navigation."""

    NO_REFERRER = "no_referrer"
    NO_REFERRER_WHEN_DOWNGRADE = "no_referrer_when_downgrade"
    ORIGIN = "origin"
    ORIGIN_WHEN_CROSS_ORIGIN = "origin_when_cross_origin"
    SAME_ORIGIN = "same_origin"
    STRICT_ORIGIN = "strict_origin"
    STRICT_ORIGIN_WHEN_CROSS_ORIGIN = "strict_origin_when_cross_origin"
    UNSAFE_URL = "unsafe_url"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class CompilationCacheParams:
    """Per-script compilation cache parameters for `Page.produceCompilationCache`"""

    # The URL of the script to produce a compilation cache entry for. # noqa
    url: str
    # A hint to the backend whether eager compilation is recommended. (theactual compilation mode used is upon backend discretion). # noqa
    eager: typing.Optional[bool]


class AutoResponseMode(str, enum.Enum):
    """Enum of possible auto-reponse for permisison / prompt dialogs."""

    NONE = "none"
    AUTO_ACCEPT = "auto_accept"
    AUTO_REJECT = "auto_reject"
    AUTO_OPT_OUT = "auto_opt_out"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class NavigationType(str, enum.Enum):
    """The type of a frameNavigated event."""

    NAVIGATION = "navigation"
    BACK_FORWARD_CACHE_RESTORE = "back_forward_cache_restore"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class BackForwardCacheNotRestoredReason(str, enum.Enum):
    """List of not restored reasons for back-forward cache."""

    NOT_PRIMARY_MAIN_FRAME = "not_primary_main_frame"
    BACK_FORWARD_CACHE_DISABLED = "back_forward_cache_disabled"
    RELATED_ACTIVE_CONTENTS_EXIST = "related_active_contents_exist"
    H_T_T_P_STATUS_NOT_O_K = "http_status_not_ok"
    SCHEME_NOT_H_T_T_P_OR_H_T_T_P_S = "scheme_not_http_or_https"
    LOADING = "loading"
    WAS_GRANTED_MEDIA_ACCESS = "was_granted_media_access"
    DISABLE_FOR_RENDER_FRAME_HOST_CALLED = "disable_for_render_frame_host_called"
    DOMAIN_NOT_ALLOWED = "domain_not_allowed"
    H_T_T_P_METHOD_NOT_G_E_T = "http_method_not_get"
    SUBFRAME_IS_NAVIGATING = "subframe_is_navigating"
    TIMEOUT = "timeout"
    CACHE_LIMIT = "cache_limit"
    JAVA_SCRIPT_EXECUTION = "java_script_execution"
    RENDERER_PROCESS_KILLED = "renderer_process_killed"
    RENDERER_PROCESS_CRASHED = "renderer_process_crashed"
    SCHEDULER_TRACKED_FEATURE_USED = "scheduler_tracked_feature_used"
    CONFLICTING_BROWSING_INSTANCE = "conflicting_browsing_instance"
    CACHE_FLUSHED = "cache_flushed"
    SERVICE_WORKER_VERSION_ACTIVATION = "service_worker_version_activation"
    SESSION_RESTORED = "session_restored"
    SERVICE_WORKER_POST_MESSAGE = "service_worker_post_message"
    ENTERED_BACK_FORWARD_CACHE_BEFORE_SERVICE_WORKER_HOST_ADDED = (
        "entered_back_forward_cache_before_service_worker_host_added"
    )
    RENDER_FRAME_HOST_REUSED__SAME_SITE = "render_frame_host_reused__same_site"
    RENDER_FRAME_HOST_REUSED__CROSS_SITE = "render_frame_host_reused__cross_site"
    SERVICE_WORKER_CLAIM = "service_worker_claim"
    IGNORE_EVENT_AND_EVICT = "ignore_event_and_evict"
    HAVE_INNER_CONTENTS = "have_inner_contents"
    TIMEOUT_PUTTING_IN_CACHE = "timeout_putting_in_cache"
    BACK_FORWARD_CACHE_DISABLED_BY_LOW_MEMORY = "back_forward_cache_disabled_by_low_memory"
    BACK_FORWARD_CACHE_DISABLED_BY_COMMAND_LINE = "back_forward_cache_disabled_by_command_line"
    NETWORK_REQUEST_DATAPIPE_DRAINED_AS_BYTES_CONSUMER = "network_request_datapipe_drained_as_bytes_consumer"
    NETWORK_REQUEST_REDIRECTED = "network_request_redirected"
    NETWORK_REQUEST_TIMEOUT = "network_request_timeout"
    NETWORK_EXCEEDS_BUFFER_LIMIT = "network_exceeds_buffer_limit"
    NAVIGATION_CANCELLED_WHILE_RESTORING = "navigation_cancelled_while_restoring"
    NOT_MOST_RECENT_NAVIGATION_ENTRY = "not_most_recent_navigation_entry"
    BACK_FORWARD_CACHE_DISABLED_FOR_PRERENDER = "back_forward_cache_disabled_for_prerender"
    USER_AGENT_OVERRIDE_DIFFERS = "user_agent_override_differs"
    FOREGROUND_CACHE_LIMIT = "foreground_cache_limit"
    BROWSING_INSTANCE_NOT_SWAPPED = "browsing_instance_not_swapped"
    BACK_FORWARD_CACHE_DISABLED_FOR_DELEGATE = "back_forward_cache_disabled_for_delegate"
    UNLOAD_HANDLER_EXISTS_IN_MAIN_FRAME = "unload_handler_exists_in_main_frame"
    UNLOAD_HANDLER_EXISTS_IN_SUB_FRAME = "unload_handler_exists_in_sub_frame"
    SERVICE_WORKER_UNREGISTRATION = "service_worker_unregistration"
    CACHE_CONTROL_NO_STORE = "cache_control_no_store"
    CACHE_CONTROL_NO_STORE_COOKIE_MODIFIED = "cache_control_no_store_cookie_modified"
    CACHE_CONTROL_NO_STORE_H_T_T_P_ONLY_COOKIE_MODIFIED = "cache_control_no_store_http_only_cookie_modified"
    NO_RESPONSE_HEAD = "no_response_head"
    UNKNOWN = "unknown"
    ACTIVATION_NAVIGATIONS_DISALLOWED_FOR_BUG1234857 = "activation_navigations_disallowed_for_bug1234857"
    ERROR_DOCUMENT = "error_document"
    FENCED_FRAMES_EMBEDDER = "fenced_frames_embedder"
    WEB_SOCKET = "web_socket"
    WEB_TRANSPORT = "web_transport"
    WEB_R_T_C = "web_rtc"
    MAIN_RESOURCE_HAS_CACHE_CONTROL_NO_STORE = "main_resource_has_cache_control_no_store"
    MAIN_RESOURCE_HAS_CACHE_CONTROL_NO_CACHE = "main_resource_has_cache_control_no_cache"
    SUBRESOURCE_HAS_CACHE_CONTROL_NO_STORE = "subresource_has_cache_control_no_store"
    SUBRESOURCE_HAS_CACHE_CONTROL_NO_CACHE = "subresource_has_cache_control_no_cache"
    CONTAINS_PLUGINS = "contains_plugins"
    DOCUMENT_LOADED = "document_loaded"
    DEDICATED_WORKER_OR_WORKLET = "dedicated_worker_or_worklet"
    OUTSTANDING_NETWORK_REQUEST_OTHERS = "outstanding_network_request_others"
    OUTSTANDING_INDEXED_D_B_TRANSACTION = "outstanding_indexed_db_transaction"
    REQUESTED_M_I_D_I_PERMISSION = "requested_midi_permission"
    REQUESTED_AUDIO_CAPTURE_PERMISSION = "requested_audio_capture_permission"
    REQUESTED_VIDEO_CAPTURE_PERMISSION = "requested_video_capture_permission"
    REQUESTED_BACK_FORWARD_CACHE_BLOCKED_SENSORS = "requested_back_forward_cache_blocked_sensors"
    REQUESTED_BACKGROUND_WORK_PERMISSION = "requested_background_work_permission"
    BROADCAST_CHANNEL = "broadcast_channel"
    INDEXED_D_B_CONNECTION = "indexed_db_connection"
    WEB_X_R = "web_xr"
    SHARED_WORKER = "shared_worker"
    WEB_LOCKS = "web_locks"
    WEB_H_I_D = "web_hid"
    WEB_SHARE = "web_share"
    REQUESTED_STORAGE_ACCESS_GRANT = "requested_storage_access_grant"
    WEB_NFC = "web_nfc"
    OUTSTANDING_NETWORK_REQUEST_FETCH = "outstanding_network_request_fetch"
    OUTSTANDING_NETWORK_REQUEST_X_H_R = "outstanding_network_request_xhr"
    APP_BANNER = "app_banner"
    PRINTING = "printing"
    WEB_DATABASE = "web_database"
    PICTURE_IN_PICTURE = "picture_in_picture"
    PORTAL = "portal"
    SPEECH_RECOGNIZER = "speech_recognizer"
    IDLE_MANAGER = "idle_manager"
    PAYMENT_MANAGER = "payment_manager"
    SPEECH_SYNTHESIS = "speech_synthesis"
    KEYBOARD_LOCK = "keyboard_lock"
    WEB_O_T_P_SERVICE = "web_otp_service"
    OUTSTANDING_NETWORK_REQUEST_DIRECT_SOCKET = "outstanding_network_request_direct_socket"
    INJECTED_JAVASCRIPT = "injected_javascript"
    INJECTED_STYLE_SHEET = "injected_style_sheet"
    KEEPALIVE_REQUEST = "keepalive_request"
    INDEXED_D_B_EVENT = "indexed_db_event"
    DUMMY = "dummy"
    AUTHORIZATION_HEADER = "authorization_header"
    CONTENT_SECURITY_HANDLER = "content_security_handler"
    CONTENT_WEB_AUTHENTICATION_A_P_I = "content_web_authentication_api"
    CONTENT_FILE_CHOOSER = "content_file_chooser"
    CONTENT_SERIAL = "content_serial"
    CONTENT_FILE_SYSTEM_ACCESS = "content_file_system_access"
    CONTENT_MEDIA_DEVICES_DISPATCHER_HOST = "content_media_devices_dispatcher_host"
    CONTENT_WEB_BLUETOOTH = "content_web_bluetooth"
    CONTENT_WEB_U_S_B = "content_web_usb"
    CONTENT_MEDIA_SESSION_SERVICE = "content_media_session_service"
    CONTENT_SCREEN_READER = "content_screen_reader"
    EMBEDDER_POPUP_BLOCKER_TAB_HELPER = "embedder_popup_blocker_tab_helper"
    EMBEDDER_SAFE_BROWSING_TRIGGERED_POPUP_BLOCKER = "embedder_safe_browsing_triggered_popup_blocker"
    EMBEDDER_SAFE_BROWSING_THREAT_DETAILS = "embedder_safe_browsing_threat_details"
    EMBEDDER_APP_BANNER_MANAGER = "embedder_app_banner_manager"
    EMBEDDER_DOM_DISTILLER_VIEWER_SOURCE = "embedder_dom_distiller_viewer_source"
    EMBEDDER_DOM_DISTILLER_SELF_DELETING_REQUEST_DELEGATE = "embedder_dom_distiller_self_deleting_request_delegate"
    EMBEDDER_OOM_INTERVENTION_TAB_HELPER = "embedder_oom_intervention_tab_helper"
    EMBEDDER_OFFLINE_PAGE = "embedder_offline_page"
    EMBEDDER_CHROME_PASSWORD_MANAGER_CLIENT_BIND_CREDENTIAL_MANAGER = (
        "embedder_chrome_password_manager_client_bind_credential_manager"
    )
    EMBEDDER_PERMISSION_REQUEST_MANAGER = "embedder_permission_request_manager"
    EMBEDDER_MODAL_DIALOG = "embedder_modal_dialog"
    EMBEDDER_EXTENSIONS = "embedder_extensions"
    EMBEDDER_EXTENSION_MESSAGING = "embedder_extension_messaging"
    EMBEDDER_EXTENSION_MESSAGING_FOR_OPEN_PORT = "embedder_extension_messaging_for_open_port"
    EMBEDDER_EXTENSION_SENT_MESSAGE_TO_CACHED_FRAME = "embedder_extension_sent_message_to_cached_frame"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class BackForwardCacheNotRestoredReasonType(str, enum.Enum):
    """Types of not restored reasons for back-forward cache."""

    SUPPORT_PENDING = "support_pending"
    PAGE_SUPPORT_NEEDED = "page_support_needed"
    CIRCUMSTANTIAL = "circumstantial"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class BackForwardCacheNotRestoredExplanation:
    """Description is missing from the devtools protocol document."""

    # Type of the reason # noqa
    type: BackForwardCacheNotRestoredReasonType
    # Not restored reason # noqa
    reason: BackForwardCacheNotRestoredReason
    # Context associated with the reason. The meaning of this context isdependent on the reason: - EmbedderExtensionSentMessageToCachedFrame: theextension ID. # noqa
    context: typing.Optional[str]


@dataclass
class BackForwardCacheNotRestoredExplanationTree:
    """Description is missing from the devtools protocol document."""

    # URL of each frame # noqa
    url: str
    # Not restored reasons of each frame # noqa
    explanations: BackForwardCacheNotRestoredExplanation
    # Array of children frame # noqa
    children: BackForwardCacheNotRestoredExplanationTree


@dataclass
@memoize_event("Page.domContentEventFired")
class DomContentEventFired:
    """Description is missing from the devtools protocol document."""

    timestamp: network.MonotonicTime


@dataclass
@memoize_event("Page.fileChooserOpened")
class FileChooserOpened:
    """Emitted only when `page.interceptFileChooser` is enabled."""

    frame_id: FrameId
    mode: typing.Literal["select_single", "select_multiple"]
    backend_node_id: typing.Optional[dom.BackendNodeId]


@dataclass
@memoize_event("Page.frameAttached")
class FrameAttached:
    """Fired when frame has been attached to its parent."""

    frame_id: FrameId
    parent_frame_id: FrameId
    stack: typing.Optional[runtime.StackTrace]


@dataclass
@memoize_event("Page.frameClearedScheduledNavigation")
class FrameClearedScheduledNavigation:
    """Fired when frame no longer has a scheduled navigation."""

    frame_id: FrameId


@dataclass
@memoize_event("Page.frameDetached")
class FrameDetached:
    """Fired when frame has been detached from its parent."""

    frame_id: FrameId
    reason: typing.Literal["remove", "swap"]


@dataclass
@memoize_event("Page.frameNavigated")
class FrameNavigated:
    """Fired once navigation of the frame has completed.

    Frame is now associated with the new loader.
    """

    frame: Frame
    type: NavigationType


@dataclass
@memoize_event("Page.documentOpened")
class DocumentOpened:
    """Fired when opening document to write to."""

    frame: Frame


@dataclass
@memoize_event("Page.frameResized")
class FrameResized:
    """Description is missing from the devtools protocol document."""


@dataclass
@memoize_event("Page.frameRequestedNavigation")
class FrameRequestedNavigation:
    """Fired when a renderer-initiated navigation is requested.

    Navigation may still be cancelled after the event is issued.
    """

    frame_id: FrameId
    reason: ClientNavigationReason
    url: str
    disposition: ClientNavigationDisposition


@dataclass
@memoize_event("Page.frameScheduledNavigation")
class FrameScheduledNavigation:
    """Fired when frame schedules a potential navigation."""

    frame_id: FrameId
    delay: float
    reason: ClientNavigationReason
    url: str


@dataclass
@memoize_event("Page.frameStartedLoading")
class FrameStartedLoading:
    """Fired when frame has started loading."""

    frame_id: FrameId


@dataclass
@memoize_event("Page.frameStoppedLoading")
class FrameStoppedLoading:
    """Fired when frame has stopped loading."""

    frame_id: FrameId


@dataclass
@memoize_event("Page.downloadWillBegin")
class DownloadWillBegin:
    """Fired when page is about to start a download.

    Deprecated. Use Browser.downloadWillBegin instead.
    """

    frame_id: FrameId
    guid: str
    url: str
    suggested_filename: str


@dataclass
@memoize_event("Page.downloadProgress")
class DownloadProgress:
    """Fired when download makes progress.

    Last call has |done| == true. Deprecated. Use Browser.downloadProgress instead.
    """

    guid: str
    total_bytes: float
    received_bytes: float
    state: typing.Literal["in_progress", "completed", "canceled"]


@dataclass
@memoize_event("Page.interstitialHidden")
class InterstitialHidden:
    """Fired when interstitial page was hidden."""


@dataclass
@memoize_event("Page.interstitialShown")
class InterstitialShown:
    """Fired when interstitial page was shown."""


@dataclass
@memoize_event("Page.javascriptDialogClosed")
class JavascriptDialogClosed:
    """Fired when a JavaScript initiated dialog (alert, confirm, prompt, or onbeforeunload) has been closed."""

    result: bool
    user_input: str


@dataclass
@memoize_event("Page.javascriptDialogOpening")
class JavascriptDialogOpening:
    """Fired when a JavaScript initiated dialog (alert, confirm, prompt, or onbeforeunload) is about to open."""

    url: str
    message: str
    type: DialogType
    has_browser_handler: bool
    default_prompt: typing.Optional[str]


@dataclass
@memoize_event("Page.lifecycleEvent")
class LifecycleEvent:
    """Fired for top level page lifecycle events such as navigation, load, paint, etc."""

    frame_id: FrameId
    loader_id: network.LoaderId
    name: str
    timestamp: network.MonotonicTime


@dataclass
@memoize_event("Page.backForwardCacheNotUsed")
class BackForwardCacheNotUsed:
    """Fired for failed bfcache history navigations if BackForwardCache feature is enabled.

    Do not assume any ordering with the Page.frameNavigated event. This event is fired only for main-frame history
    navigation where the document changes (non-same-document navigations), when bfcache navigation fails.
    """

    loader_id: network.LoaderId
    frame_id: FrameId
    not_restored_explanations: typing.List[BackForwardCacheNotRestoredExplanation]
    not_restored_explanations_tree: typing.Optional[BackForwardCacheNotRestoredExplanationTree]


@dataclass
@memoize_event("Page.loadEventFired")
class LoadEventFired:
    """Description is missing from the devtools protocol document."""

    timestamp: network.MonotonicTime


@dataclass
@memoize_event("Page.navigatedWithinDocument")
class NavigatedWithinDocument:
    """Fired when same-document navigation happens, e.g. due to history API usage or anchor navigation."""

    frame_id: FrameId
    url: str


@dataclass
@memoize_event("Page.screencastFrame")
class ScreencastFrame:
    """Compressed image data requested by the `startScreencast`."""

    data: str
    metadata: ScreencastFrameMetadata
    session_id: int


@dataclass
@memoize_event("Page.screencastVisibilityChanged")
class ScreencastVisibilityChanged:
    """Fired when the page with currently enabled screencast was shown or hidden `."""

    visible: bool


@dataclass
@memoize_event("Page.windowOpen")
class WindowOpen:
    """Fired when a new window is going to be opened, via window.open(), link click, form submission, etc."""

    url: str
    window_name: str
    window_features: typing.List[str]
    user_gesture: bool


@dataclass
@memoize_event("Page.compilationCacheProduced")
class CompilationCacheProduced:
    """Issued for every compilation cache generated.

    Is only available if Page.setGenerateCompilationCache is enabled.
    """

    url: str
    data: str


async def add_script_to_evaluate_on_load() -> None:
    """Deprecated, please use addScriptToEvaluateOnNewDocument instead.

    # noqa
    """
    ...


async def add_script_to_evaluate_on_new_document() -> None:
    """Evaluates given script in every frame upon creation (before loading frame's scripts).

    # noqa
    """
    ...


async def bring_to_front() -> None:
    """Brings page to front (activates tab).

    # noqa
    """
    ...


async def capture_screenshot() -> None:
    """Capture page screenshot.

    # noqa
    """
    ...


async def capture_snapshot() -> None:
    """Returns a snapshot of the page as a string.

    For MHTML format, the serialization includes iframes, shadow DOM, external resources, and element-inline styles. #
    noqa
    """
    ...


async def clear_device_metrics_override() -> None:
    """Clears the overridden device metrics.

    # noqa
    """
    ...


async def clear_device_orientation_override() -> None:
    """Clears the overridden Device Orientation.

    # noqa
    """
    ...


async def clear_geolocation_override() -> None:
    """Clears the overridden Geolocation Position and Error.

    # noqa
    """
    ...


async def create_isolated_world() -> None:
    """Creates an isolated world for the given frame.

    # noqa
    """
    ...


async def delete_cookie() -> None:
    """Deletes browser cookie with given name, domain and path.

    # noqa
    """
    ...


async def disable() -> None:
    """Disables page domain notifications.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables page domain notifications.

    # noqa
    """
    ...


async def get_app_manifest() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def get_installability_errors() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def get_manifest_icons() -> None:
    """Deprecated because it's not guaranteed that the returned icon is in fact the one used for PWA installation.

    # noqa
    """
    ...


async def get_app_id() -> None:
    """Returns the unique (PWA) app id.

    Only returns values if the feature flag 'WebAppEnableManifestId' is enabled # noqa
    """
    ...


async def get_ad_script_id() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def get_cookies() -> None:
    """Returns all browser cookies for the page and all of its subframes.

    Depending on the backend support, will return detailed cookie information in the `cookies` field. # noqa
    """
    ...


async def get_frame_tree() -> None:
    """Returns present frame tree structure.

    # noqa
    """
    ...


async def get_layout_metrics() -> None:
    """Returns metrics relating to the layouting of the page, such as viewport bounds/scale.

    # noqa
    """
    ...


async def get_navigation_history() -> None:
    """Returns navigation history for the current page.

    # noqa
    """
    ...


async def reset_navigation_history() -> None:
    """Resets navigation history for the current page.

    # noqa
    """
    ...


async def get_resource_content() -> None:
    """Returns content of the given resource.

    # noqa
    """
    ...


async def get_resource_tree() -> None:
    """Returns present frame / resource tree structure.

    # noqa
    """
    ...


async def handle_java_script_dialog() -> None:
    """Accepts or dismisses a JavaScript initiated dialog (alert, confirm, prompt, or onbeforeunload).

    # noqa
    """
    ...


async def navigate() -> None:
    """Navigates current page to the given URL.

    # noqa
    """
    ...


async def navigate_to_history_entry() -> None:
    """Navigates current page to the given history entry.

    # noqa
    """
    ...


async def print_to_pdf() -> None:
    """Print page as PDF.

    # noqa
    """
    ...


async def reload() -> None:
    """Reloads given page optionally ignoring the cache.

    # noqa
    """
    ...


async def remove_script_to_evaluate_on_load() -> None:
    """Deprecated, please use removeScriptToEvaluateOnNewDocument instead.

    # noqa
    """
    ...


async def remove_script_to_evaluate_on_new_document() -> None:
    """Removes given script from the list.

    # noqa
    """
    ...


async def screencast_frame_ack() -> None:
    """Acknowledges that a screencast frame has been received by the frontend.

    # noqa
    """
    ...


async def search_in_resource() -> None:
    """Searches for given string in resource content.

    # noqa
    """
    ...


async def set_ad_blocking_enabled() -> None:
    """Enable Chrome's experimental ad filter on all sites.

    # noqa
    """
    ...


async def set_bypass_csp() -> None:
    """Enable page Content Security Policy by-passing.

    # noqa
    """
    ...


async def get_permissions_policy_state() -> None:
    """Get Permissions Policy state on given frame.

    # noqa
    """
    ...


async def get_origin_trials() -> None:
    """Get Origin Trials on given frame.

    # noqa
    """
    ...


async def set_device_metrics_override() -> None:
    """Overrides the values of device screen dimensions (window.screen.width, window.screen.height, window.innerWidth,
    window.innerHeight, and "device-width"/"device-height"-related CSS media query results).

    # noqa
    """
    ...


async def set_device_orientation_override() -> None:
    """Overrides the Device Orientation.

    # noqa
    """
    ...


async def set_font_families() -> None:
    """Set generic font families.

    # noqa
    """
    ...


async def set_font_sizes() -> None:
    """Set default font sizes.

    # noqa
    """
    ...


async def set_document_content() -> None:
    """Sets given markup as the document's HTML.

    # noqa
    """
    ...


async def set_download_behavior() -> None:
    """Set the behavior when downloading a file.

    # noqa
    """
    ...


async def set_geolocation_override() -> None:
    """Overrides the Geolocation Position or Error.

    Omitting any of the parameters emulates position unavailable. # noqa
    """
    ...


async def set_lifecycle_events_enabled() -> None:
    """Controls whether page will emit lifecycle events.

    # noqa
    """
    ...


async def set_touch_emulation_enabled() -> None:
    """Toggles mouse event-based touch event emulation.

    # noqa
    """
    ...


async def start_screencast() -> None:
    """Starts sending each frame using the `screencastFrame` event.

    # noqa
    """
    ...


async def stop_loading() -> None:
    """Force the page stop all navigations and pending resource fetches.

    # noqa
    """
    ...


async def crash() -> None:
    """Crashes renderer on the IO thread, generates minidumps.

    # noqa
    """
    ...


async def close() -> None:
    """Tries to close page, running its beforeunload hooks, if any.

    # noqa
    """
    ...


async def set_web_lifecycle_state() -> None:
    """Tries to update the web lifecycle state of the page.

    It will transition the page to the given state according to:
    https://github.com/WICG/web-lifecycle/ # noqa
    """
    ...


async def stop_screencast() -> None:
    """Stops sending each frame in the `screencastFrame`.

    # noqa
    """
    ...


async def produce_compilation_cache() -> None:
    """Requests backend to produce compilation cache for the specified scripts.

    `scripts` are appeneded to the list of scripts for which the cache would be produced. The list may be reset during
    page navigation. When script with a matching URL is encountered, the cache is optionally produced upon backend
    discretion, based on internal heuristics. See also: `Page.compilationCacheProduced`. # noqa
    """
    ...


async def add_compilation_cache() -> None:
    """Seeds compilation cache for given url.

    Compilation cache does not survive cross-process navigation. # noqa
    """
    ...


async def clear_compilation_cache() -> None:
    """Clears seeded compilation cache.

    # noqa
    """
    ...


async def set_spc_transaction_mode() -> None:
    """Sets the Secure Payment Confirmation transaction mode.

    https://w3c.github.io/secure-payment-confirmation/#sctn-automation-set-spc-transaction-mode # noqa
    """
    ...


async def set_rph_registration_mode() -> None:
    """Extensions for Custom Handlers API:

    https://html.spec.whatwg.org/multipage/system-state.html#rph-automation # noqa
    """
    ...


async def generate_test_report() -> None:
    """Generates a report for testing.

    # noqa
    """
    ...


async def wait_for_debugger() -> None:
    """Pauses page execution.

    Can be resumed using generic Runtime.runIfWaitingForDebugger. # noqa
    """
    ...


async def set_intercept_file_chooser_dialog() -> None:
    """Intercept file chooser requests and transfer control to protocol clients.

    When file chooser interception is enabled, native file chooser dialog is not shown. Instead, a protocol event
    `Page.fileChooserOpened` is emitted. # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Emulation`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Emulation/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from .utils import memoize_event


@dataclass
class ScreenOrientation:
    """Screen orientation."""

    # Orientation type. # noqa
    type: typing.List[typing.Literal["portraitPrimary", "portraitSecondary", "landscapePrimary", "landscapeSecondary"]]
    # Orientation angle. # noqa
    angle: int


@dataclass
class DisplayFeature:
    """Description is missing from the devtools protocol document."""

    # Orientation of a display feature in relation to screen # noqa
    orientation: typing.List[typing.Literal["vertical", "horizontal"]]
    # The offset from the screen origin in either the x (for verticalorientation) or y (for horizontal orientation) direction. # noqa
    offset: int
    # A display feature may mask content such that it is not physicallydisplayed - this length along with the offset describes this area. A displayfeature that only splits content will have a 0 mask_length. # noqa
    mask_length: int


@dataclass
class MediaFeature:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    name: str
    # Description is missing from the devtools protocol document. # noqa
    value: str


class VirtualTimePolicy(str, enum.Enum):
    """advance: If the scheduler runs out of immediate work, the virtual time base may fast forward to
    allow the next delayed task (if any) to run; pause: The virtual time base may not advance;
    pauseIfNetworkFetchesPending: The virtual time base may not advance if there are any pending
    resource fetches."""

    ADVANCE = "advance"
    PAUSE = "pause"
    PAUSE_IF_NETWORK_FETCHES_PENDING = "pause_if_network_fetches_pending"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class UserAgentBrandVersion:
    """Used to specify User Agent Cient Hints to emulate.

    See https://wicg.github.io/ua-client-hints
    """

    # Description is missing from the devtools protocol document. # noqa
    brand: str
    # Description is missing from the devtools protocol document. # noqa
    version: str


@dataclass
class UserAgentMetadata:
    """Used to specify User Agent Cient Hints to emulate.

    See https://wicg.github.io/ua-client-hints
    Missing optional values will be filled in by the target with what it would normally use.
    """

    # Description is missing from the devtools protocol document. # noqa
    platform: str
    # Description is missing from the devtools protocol document. # noqa
    platform_version: str
    # Description is missing from the devtools protocol document. # noqa
    architecture: str
    # Description is missing from the devtools protocol document. # noqa
    model: str
    # Description is missing from the devtools protocol document. # noqa
    mobile: bool
    # Brands appearing in Sec-CH-UA. # noqa
    brands: typing.Optional[UserAgentBrandVersion]
    # Brands appearing in Sec-CH-UA-Full-Version-List. # noqa
    full_version_list: typing.Optional[UserAgentBrandVersion]
    # Description is missing from the devtools protocol document. # noqa
    full_version: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    bitness: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    wow64: typing.Optional[bool]


class DisabledImageType(str, enum.Enum):
    """Enum of image types that can be disabled."""

    AVIF = "avif"
    WEBP = "webp"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
@memoize_event("Emulation.virtualTimeBudgetExpired")
class VirtualTimeBudgetExpired:
    """Notification sent after the virtual time budget for the current VirtualTimePolicy has run out."""


async def can_emulate() -> None:
    """Tells whether emulation is supported.

    # noqa
    """
    ...


async def clear_device_metrics_override() -> None:
    """Clears the overridden device metrics.

    # noqa
    """
    ...


async def clear_geolocation_override() -> None:
    """Clears the overridden Geolocation Position and Error.

    # noqa
    """
    ...


async def reset_page_scale_factor() -> None:
    """Requests that page scale factor is reset to initial values.

    # noqa
    """
    ...


async def set_focus_emulation_enabled() -> None:
    """Enables or disables simulating a focused and active page.

    # noqa
    """
    ...


async def set_auto_dark_mode_override() -> None:
    """Automatically render all web contents using a dark theme.

    # noqa
    """
    ...


async def set_cpu_throttling_rate() -> None:
    """Enables CPU throttling to emulate slow CPUs.

    # noqa
    """
    ...


async def set_default_background_color_override() -> None:
    """Sets or clears an override of the default background color of the frame.

    This override is used if the content does not specify one. # noqa
    """
    ...


async def set_device_metrics_override() -> None:
    """Overrides the values of device screen dimensions (window.screen.width, window.screen.height, window.innerWidth,
    window.innerHeight, and "device-width"/"device-height"-related CSS media query results).

    # noqa
    """
    ...


async def set_scrollbars_hidden() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def set_document_cookie_disabled() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def set_emit_touch_events_for_mouse() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def set_emulated_media() -> None:
    """Emulates the given media type or media feature for CSS media queries.

    # noqa
    """
    ...


async def set_emulated_vision_deficiency() -> None:
    """Emulates the given vision deficiency.

    # noqa
    """
    ...


async def set_geolocation_override() -> None:
    """Overrides the Geolocation Position or Error.

    Omitting any of the parameters emulates position unavailable. # noqa
    """
    ...


async def set_idle_override() -> None:
    """Overrides the Idle state.

    # noqa
    """
    ...


async def clear_idle_override() -> None:
    """Clears Idle state overrides.

    # noqa
    """
    ...


async def set_navigator_overrides() -> None:
    """Overrides value returned by the javascript navigator object.

    # noqa
    """
    ...


async def set_page_scale_factor() -> None:
    """Sets a specified page scale factor.

    # noqa
    """
    ...


async def set_script_execution_disabled() -> None:
    """Switches script execution in the page.

    # noqa
    """
    ...


async def set_touch_emulation_enabled() -> None:
    """Enables touch on platforms which do not support them.

    # noqa
    """
    ...


async def set_virtual_time_policy() -> None:
    """Turns on virtual time for all frames (replacing real-time with a synthetic time source) and sets the current
    virtual time policy.

    Note this supersedes any previous time budget. # noqa
    """
    ...


async def set_locale_override() -> None:
    """Overrides default host system locale with the specified one.

    # noqa
    """
    ...


async def set_timezone_override() -> None:
    """Overrides default host system timezone with the specified one.

    # noqa
    """
    ...


async def set_visible_size() -> None:
    """Resizes the frame/viewport of the page.

    Note that this does not affect the frame's container (e.g. browser window). Can be used to produce screenshots of
    the specified size. Not supported on Android. # noqa
    """
    ...


async def set_disabled_image_types() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def set_hardware_concurrency_override() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def set_user_agent_override() -> None:
    """Allows overriding user agent with the given string.

    # noqa
    """
    ...


async def set_automation_override() -> None:
    """Allows overriding the automation flag.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `IO`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/IO/

from __future__ import annotations


class StreamHandle(str):
    """This is either obtained from another method or specified as `blob:&lt;uuid&gt;` where `&lt;uuid&gt` is an UUID of
    a Blob."""

    def to_json(self) -> StreamHandle:
        return self

    @classmethod
    def from_json(cls, value: str) -> StreamHandle:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


async def close() -> None:
    """Close the stream, discard any temporary backing storage.

    # noqa
    """
    ...


async def read() -> None:
    """Read a chunk of the stream # noqa."""
    ...


async def resolve_blob() -> None:
    """Return UUID of Blob object specified by a remote object id.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `SystemInfo`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/SystemInfo/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass


@dataclass
class GPUDevice:
    """Describes a single graphics processor (GPU)."""

    # PCI ID of the GPU vendor, if available; 0 otherwise. # noqa
    vendor_id: float
    # PCI ID of the GPU device, if available; 0 otherwise. # noqa
    device_id: float
    # String description of the GPU vendor, if the PCI ID is not available. # noqa
    vendor_string: str
    # String description of the GPU device, if the PCI ID is not available. # noqa
    device_string: str
    # String description of the GPU driver vendor. # noqa
    driver_vendor: str
    # String description of the GPU driver version. # noqa
    driver_version: str
    # Sub sys ID of the GPU, only available on Windows. # noqa
    sub_sys_id: typing.Optional[float]
    # Revision of the GPU, only available on Windows. # noqa
    revision: typing.Optional[float]


@dataclass
class Size:
    """Describes the width and height dimensions of an entity."""

    # Width in pixels. # noqa
    width: int
    # Height in pixels. # noqa
    height: int


@dataclass
class VideoDecodeAcceleratorCapability:
    """Describes a supported video decoding profile with its associated minimum and maximum resolutions."""

    # Video codec profile that is supported, e.g. VP9 Profile 2. # noqa
    profile: str
    # Maximum video dimensions in pixels supported for this |profile|. # noqa
    max_resolution: Size
    # Minimum video dimensions in pixels supported for this |profile|. # noqa
    min_resolution: Size


@dataclass
class VideoEncodeAcceleratorCapability:
    """Describes a supported video encoding profile with its associated maximum resolution and maximum framerate."""

    # Video codec profile that is supported, e.g H264 Main. # noqa
    profile: str
    # Maximum video dimensions in pixels supported for this |profile|. # noqa
    max_resolution: Size
    # Maximum encoding framerate in frames per second supported for this|profile|, as fraction's numerator and denominator, e.g. 24/1 fps, 24000/1001fps, etc. # noqa
    max_framerate_numerator: int
    # Description is missing from the devtools protocol document. # noqa
    max_framerate_denominator: int


class SubsamplingFormat(str, enum.Enum):
    """YUV subsampling type of the pixels of a given image."""

    YUV420 = "yuv420"
    YUV422 = "yuv422"
    YUV444 = "yuv444"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class ImageType(str, enum.Enum):
    """Image format of a given image."""

    JPEG = "jpeg"
    WEBP = "webp"
    UNKNOWN = "unknown"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class ImageDecodeAcceleratorCapability:
    """Describes a supported image decoding profile with its associated minimum and maximum resolutions and
    subsampling."""

    # Image coded, e.g. Jpeg. # noqa
    image_type: ImageType
    # Maximum supported dimensions of the image in pixels. # noqa
    max_dimensions: Size
    # Minimum supported dimensions of the image in pixels. # noqa
    min_dimensions: Size
    # Optional array of supported subsampling formats, e.g. 4:2:0, if known. # noqa
    subsamplings: SubsamplingFormat


@dataclass
class GPUInfo:
    """Provides information about the GPU(s) on the system."""

    # The graphics devices on the system. Element 0 is the primary GPU. # noqa
    devices: GPUDevice
    # An optional array of GPU driver bug workarounds. # noqa
    driver_bug_workarounds: str
    # Supported accelerated video decoding capabilities. # noqa
    video_decoding: VideoDecodeAcceleratorCapability
    # Supported accelerated video encoding capabilities. # noqa
    video_encoding: VideoEncodeAcceleratorCapability
    # Supported accelerated image decoding capabilities. # noqa
    image_decoding: ImageDecodeAcceleratorCapability
    # An optional dictionary of additional GPU related attributes. # noqa
    aux_attributes: typing.Any
    # An optional dictionary of graphics features and their status. # noqa
    feature_status: typing.Any


@dataclass
class ProcessInfo:
    """Represents process info."""

    # Specifies process type. # noqa
    type: str
    # Specifies process id. # noqa
    id: int
    # Specifies cumulative CPU usage in seconds across all threads of theprocess since the process start. # noqa
    cpu_time: float


async def get_info() -> None:
    """Returns information about the system.

    # noqa
    """
    ...


async def get_feature_state() -> None:
    """Returns information about the feature state.

    # noqa
    """
    ...


async def get_process_info() -> None:
    """Returns information about all running processes.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `DeviceAccess`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/DeviceAccess/

from __future__ import annotations

import typing
from dataclasses import dataclass

from .utils import memoize_event


class RequestId(str):
    """Device request id."""

    def to_json(self) -> RequestId:
        return self

    @classmethod
    def from_json(cls, value: str) -> RequestId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class DeviceId(str):
    """A device id."""

    def to_json(self) -> DeviceId:
        return self

    @classmethod
    def from_json(cls, value: str) -> DeviceId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class PromptDevice:
    """Device information displayed in a user prompt to select a device."""

    # Description is missing from the devtools protocol document. # noqa
    id: DeviceId
    # Display name as it appears in a device request user prompt. # noqa
    name: str


@dataclass
@memoize_event("DeviceAccess.deviceRequestPrompted")
class DeviceRequestPrompted:
    """A device request opened a user prompt to select a device.

    Respond with the selectPrompt or cancelPrompt command.
    """

    id: RequestId
    devices: typing.List[PromptDevice]


async def enable() -> None:
    """Enable events in this domain.

    # noqa
    """
    ...


async def disable() -> None:
    """Disable events in this domain.

    # noqa
    """
    ...


async def select_prompt() -> None:
    """Select a device in response to a DeviceAccess.deviceRequestPrompted event.

    # noqa
    """
    ...


async def cancel_prompt() -> None:
    """Cancel a prompt in response to a DeviceAccess.deviceRequestPrompted event.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Storage`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Storage/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from . import network
from . import page
from .utils import memoize_event


class SerializedStorageKey(str):
    """Description is missing from the devtools protocol document."""

    def to_json(self) -> SerializedStorageKey:
        return self

    @classmethod
    def from_json(cls, value: str) -> SerializedStorageKey:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class StorageType(str, enum.Enum):
    """Enum of possible storage types."""

    APPCACHE = "appcache"
    COOKIES = "cookies"
    FILE_SYSTEMS = "file_systems"
    INDEXEDDB = "indexeddb"
    LOCAL_STORAGE = "local_storage"
    SHADER_CACHE = "shader_cache"
    WEBSQL = "websql"
    SERVICE_WORKERS = "service_workers"
    CACHE_STORAGE = "cache_storage"
    INTEREST_GROUPS = "interest_groups"
    SHARED_STORAGE = "shared_storage"
    STORAGE_BUCKETS = "storage_buckets"
    ALL = "all"
    OTHER = "other"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class UsageForType:
    """Usage for a storage type."""

    # Name of storage type. # noqa
    storage_type: StorageType
    # Storage usage (bytes). # noqa
    usage: float


@dataclass
class TrustTokens:
    """Pair of issuer origin and number of available (signed, but not used) Trust Tokens from that issuer."""

    # Description is missing from the devtools protocol document. # noqa
    issuer_origin: str
    # Description is missing from the devtools protocol document. # noqa
    count: float


class InterestGroupAccessType(str, enum.Enum):
    """Enum of interest group access types."""

    JOIN = "join"
    LEAVE = "leave"
    UPDATE = "update"
    LOADED = "loaded"
    BID = "bid"
    WIN = "win"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class InterestGroupAd:
    """Ad advertising element inside an interest group."""

    # Description is missing from the devtools protocol document. # noqa
    render_url: str
    # Description is missing from the devtools protocol document. # noqa
    metadata: typing.Optional[str]


@dataclass
class InterestGroupDetails:
    """The full details of an interest group."""

    # Description is missing from the devtools protocol document. # noqa
    owner_origin: str
    # Description is missing from the devtools protocol document. # noqa
    name: str
    # Description is missing from the devtools protocol document. # noqa
    expiration_time: network.TimeSinceEpoch
    # Description is missing from the devtools protocol document. # noqa
    joining_origin: str
    # Description is missing from the devtools protocol document. # noqa
    trusted_bidding_signals_keys: str
    # Description is missing from the devtools protocol document. # noqa
    ads: InterestGroupAd
    # Description is missing from the devtools protocol document. # noqa
    ad_components: InterestGroupAd
    # Description is missing from the devtools protocol document. # noqa
    bidding_url: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    bidding_wasm_helper_url: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    update_url: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    trusted_bidding_signals_url: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    user_bidding_signals: typing.Optional[str]


class SharedStorageAccessType(str, enum.Enum):
    """Enum of shared storage access types."""

    DOCUMENT_ADD_MODULE = "document_add_module"
    DOCUMENT_SELECT_U_R_L = "document_select_url"
    DOCUMENT_RUN = "document_run"
    DOCUMENT_SET = "document_set"
    DOCUMENT_APPEND = "document_append"
    DOCUMENT_DELETE = "document_delete"
    DOCUMENT_CLEAR = "document_clear"
    WORKLET_SET = "worklet_set"
    WORKLET_APPEND = "worklet_append"
    WORKLET_DELETE = "worklet_delete"
    WORKLET_CLEAR = "worklet_clear"
    WORKLET_GET = "worklet_get"
    WORKLET_KEYS = "worklet_keys"
    WORKLET_ENTRIES = "worklet_entries"
    WORKLET_LENGTH = "worklet_length"
    WORKLET_REMAINING_BUDGET = "worklet_remaining_budget"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class SharedStorageEntry:
    """Struct for a single key-value pair in an origin's shared storage."""

    # Description is missing from the devtools protocol document. # noqa
    key: str
    # Description is missing from the devtools protocol document. # noqa
    value: str


@dataclass
class SharedStorageMetadata:
    """Details for an origin's shared storage."""

    # Description is missing from the devtools protocol document. # noqa
    creation_time: network.TimeSinceEpoch
    # Description is missing from the devtools protocol document. # noqa
    length: int
    # Description is missing from the devtools protocol document. # noqa
    remaining_budget: float


@dataclass
class SharedStorageReportingMetadata:
    """Pair of reporting metadata details for a candidate URL for `selectURL()`."""

    # Description is missing from the devtools protocol document. # noqa
    event_type: str
    # Description is missing from the devtools protocol document. # noqa
    reporting_url: str


@dataclass
class SharedStorageUrlWithMetadata:
    """Bundles a candidate URL with its reporting metadata."""

    # Spec of candidate URL. # noqa
    url: str
    # Any associated reporting metadata. # noqa
    reporting_metadata: SharedStorageReportingMetadata


@dataclass
class SharedStorageAccessParams:
    """Bundles the parameters for shared storage access events whose presence/absence can vary according to
    SharedStorageAccessType."""

    # Spec of the module script URL. Present only forSharedStorageAccessType.documentAddModule. # noqa
    script_source_url: typing.Optional[str]
    # Name of the registered operation to be run. Present only forSharedStorageAccessType.documentRun andSharedStorageAccessType.documentSelectURL. # noqa
    operation_name: typing.Optional[str]
    # The operation's serialized data in bytes (converted to a string). Presentonly for SharedStorageAccessType.documentRun andSharedStorageAccessType.documentSelectURL. # noqa
    serialized_data: typing.Optional[str]
    # Array of candidate URLs' specs, along with any associated metadata.Present only for SharedStorageAccessType.documentSelectURL. # noqa
    urls_with_metadata: typing.Optional[SharedStorageUrlWithMetadata]
    # Key for a specific entry in an origin's shared storage. Present only forSharedStorageAccessType.documentSet, SharedStorageAccessType.documentAppend,SharedStorageAccessType.documentDelete, SharedStorageAccessType.workletSet,SharedStorageAccessType.workletAppend, SharedStorageAccessType.workletDelete,and SharedStorageAccessType.workletGet. # noqa
    key: typing.Optional[str]
    # Value for a specific entry in an origin's shared storage. Present only forSharedStorageAccessType.documentSet, SharedStorageAccessType.documentAppend,SharedStorageAccessType.workletSet, and SharedStorageAccessType.workletAppend. # noqa
    value: typing.Optional[str]
    # Whether or not to set an entry for a key if that key is already present.Present only for SharedStorageAccessType.documentSet andSharedStorageAccessType.workletSet. # noqa
    ignore_if_present: typing.Optional[bool]


class StorageBucketsDurability(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    RELAXED = "relaxed"
    STRICT = "strict"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class StorageBucketInfo:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    storage_key: SerializedStorageKey
    # Description is missing from the devtools protocol document. # noqa
    id: str
    # Description is missing from the devtools protocol document. # noqa
    name: str
    # Description is missing from the devtools protocol document. # noqa
    is_default: bool
    # Description is missing from the devtools protocol document. # noqa
    expiration: network.TimeSinceEpoch
    # Storage quota (bytes). # noqa
    quota: float
    # Description is missing from the devtools protocol document. # noqa
    persistent: bool
    # Description is missing from the devtools protocol document. # noqa
    durability: StorageBucketsDurability


@dataclass
@memoize_event("Storage.cacheStorageContentUpdated")
class CacheStorageContentUpdated:
    """A cache's contents have been modified."""

    origin: str
    storage_key: str
    cache_name: str


@dataclass
@memoize_event("Storage.cacheStorageListUpdated")
class CacheStorageListUpdated:
    """A cache has been added/deleted."""

    origin: str
    storage_key: str


@dataclass
@memoize_event("Storage.indexedDBContentUpdated")
class IndexedDBContentUpdated:
    """The origin's IndexedDB object store has been modified."""

    origin: str
    storage_key: str
    database_name: str
    object_store_name: str


@dataclass
@memoize_event("Storage.indexedDBListUpdated")
class IndexedDBListUpdated:
    """The origin's IndexedDB database list has been modified."""

    origin: str
    storage_key: str


@dataclass
@memoize_event("Storage.interestGroupAccessed")
class InterestGroupAccessed:
    """One of the interest groups was accessed by the associated page."""

    access_time: network.TimeSinceEpoch
    type: InterestGroupAccessType
    owner_origin: str
    name: str


@dataclass
@memoize_event("Storage.sharedStorageAccessed")
class SharedStorageAccessed:
    """Shared storage was accessed by the associated page.

    The following parameters are included in all events.
    """

    access_time: network.TimeSinceEpoch
    type: SharedStorageAccessType
    main_frame_id: page.FrameId
    owner_origin: str
    params: SharedStorageAccessParams


@dataclass
@memoize_event("Storage.storageBucketCreatedOrUpdated")
class StorageBucketCreatedOrUpdated:
    """Description is missing from the devtools protocol document."""

    bucket: StorageBucketInfo


@dataclass
@memoize_event("Storage.storageBucketDeleted")
class StorageBucketDeleted:
    """Description is missing from the devtools protocol document."""

    bucket_id: str


async def get_storage_key_for_frame() -> None:
    """Returns a storage key given a frame id.

    # noqa
    """
    ...


async def clear_data_for_origin() -> None:
    """Clears storage for origin.

    # noqa
    """
    ...


async def clear_data_for_storage_key() -> None:
    """Clears storage for storage key.

    # noqa
    """
    ...


async def get_cookies() -> None:
    """Returns all browser cookies.

    # noqa
    """
    ...


async def set_cookies() -> None:
    """Sets given cookies.

    # noqa
    """
    ...


async def clear_cookies() -> None:
    """Clears cookies.

    # noqa
    """
    ...


async def get_usage_and_quota() -> None:
    """Returns usage and quota in bytes.

    # noqa
    """
    ...


async def override_quota_for_origin() -> None:
    """Override quota for the specified origin # noqa."""
    ...


async def track_cache_storage_for_origin() -> None:
    """Registers origin to be notified when an update occurs to its cache storage list.

    # noqa
    """
    ...


async def track_cache_storage_for_storage_key() -> None:
    """Registers storage key to be notified when an update occurs to its cache storage list.

    # noqa
    """
    ...


async def track_indexed_db_for_origin() -> None:
    """Registers origin to be notified when an update occurs to its IndexedDB.

    # noqa
    """
    ...


async def track_indexed_db_for_storage_key() -> None:
    """Registers storage key to be notified when an update occurs to its IndexedDB.

    # noqa
    """
    ...


async def untrack_cache_storage_for_origin() -> None:
    """Unregisters origin from receiving notifications for cache storage.

    # noqa
    """
    ...


async def untrack_cache_storage_for_storage_key() -> None:
    """Unregisters storage key from receiving notifications for cache storage.

    # noqa
    """
    ...


async def untrack_indexed_db_for_origin() -> None:
    """Unregisters origin from receiving notifications for IndexedDB.

    # noqa
    """
    ...


async def untrack_indexed_db_for_storage_key() -> None:
    """Unregisters storage key from receiving notifications for IndexedDB.

    # noqa
    """
    ...


async def get_trust_tokens() -> None:
    """Returns the number of stored Trust Tokens per issuer for the current browsing context.

    # noqa
    """
    ...


async def clear_trust_tokens() -> None:
    """Removes all Trust Tokens issued by the provided issuerOrigin.

    Leaves other stored data, including the issuer's Redemption Records, intact. # noqa
    """
    ...


async def get_interest_group_details() -> None:
    """Gets details for a named interest group.

    # noqa
    """
    ...


async def set_interest_group_tracking() -> None:
    """Enables/Disables issuing of interestGroupAccessed events.

    # noqa
    """
    ...


async def get_shared_storage_metadata() -> None:
    """Gets metadata for an origin's shared storage.

    # noqa
    """
    ...


async def get_shared_storage_entries() -> None:
    """Gets the entries in an given origin's shared storage.

    # noqa
    """
    ...


async def set_shared_storage_entry() -> None:
    """Sets entry with `key` and `value` for a given origin's shared storage.

    # noqa
    """
    ...


async def delete_shared_storage_entry() -> None:
    """Deletes entry for `key` (if it exists) for a given origin's shared storage.

    # noqa
    """
    ...


async def clear_shared_storage_entries() -> None:
    """Clears all entries for a given origin's shared storage.

    # noqa
    """
    ...


async def reset_shared_storage_budget() -> None:
    """Resets the budget for `ownerOrigin` by clearing all budget withdrawals.

    # noqa
    """
    ...


async def set_shared_storage_tracking() -> None:
    """Enables/disables issuing of sharedStorageAccessed events.

    # noqa
    """
    ...


async def set_storage_bucket_tracking() -> None:
    """Set tracking for a storage key's buckets.

    # noqa
    """
    ...


async def delete_storage_bucket() -> None:
    """Deletes the Storage Bucket with the given storage key and bucket name.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Fetch`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Fetch/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from . import network
from . import page
from .utils import memoize_event


class RequestId(str):
    """Unique request identifier."""

    def to_json(self) -> RequestId:
        return self

    @classmethod
    def from_json(cls, value: str) -> RequestId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class RequestStage(str, enum.Enum):
    """Stages of the request to handle.

    Request will intercept before the request is sent. Response will intercept after the response is received (but
    before response body is received).
    """

    REQUEST = "request"
    RESPONSE = "response"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class RequestPattern:
    """Description is missing from the devtools protocol document."""

    # Wildcards (`'*'` -> zero or more, `'?'` -> exactly one) are allowed.Escape character is backslash. Omitting is equivalent to `"*"`. # noqa
    url_pattern: typing.Optional[str]
    # If set, only requests for matching resource types will be intercepted. # noqa
    resource_type: typing.Optional[network.ResourceType]
    # Stage at which to begin intercepting requests. Default is Request. # noqa
    request_stage: typing.Optional[RequestStage]


@dataclass
class HeaderEntry:
    """Response HTTP header entry."""

    # Description is missing from the devtools protocol document. # noqa
    name: str
    # Description is missing from the devtools protocol document. # noqa
    value: str


@dataclass
class AuthChallenge:
    """Authorization challenge for HTTP status code 401 or 407."""

    # Origin of the challenger. # noqa
    origin: str
    # The authentication scheme used, such as basic or digest # noqa
    scheme: str
    # The realm of the challenge. May be empty. # noqa
    realm: str
    # Source of the authentication challenge. # noqa
    source: typing.Optional[typing.List[typing.Literal["Server", "Proxy"]]]


@dataclass
class AuthChallengeResponse:
    """Response to an AuthChallenge."""

    # The decision on what to do in response to the authorization challenge.Default means deferring to the default behavior of the net stack, which willlikely either the Cancel authentication or display a popup dialog box. # noqa
    response: typing.List[typing.Literal["Default", "CancelAuth", "ProvideCredentials"]]
    # The username to provide, possibly empty. Should only be set if response isProvideCredentials. # noqa
    username: typing.Optional[str]
    # The password to provide, possibly empty. Should only be set if response isProvideCredentials. # noqa
    password: typing.Optional[str]


@dataclass
@memoize_event("Fetch.requestPaused")
class RequestPaused:
    """Issued when the domain is enabled and the request URL matches the specified filter.

    The request is paused until the client responds with one of continueRequest, failRequest or fulfillRequest. The
    stage of the request can be determined by presence of responseErrorReason and responseStatusCode -- the request is
    at the response stage if either of these fields is present and in the request stage otherwise.
    """

    request_id: RequestId
    request: network.Request
    frame_id: page.FrameId
    resource_type: network.ResourceType
    response_error_reason: typing.Optional[network.ErrorReason]
    response_status_code: typing.Optional[int]
    response_status_text: typing.Optional[str]
    response_headers: typing.Optional[typing.List[HeaderEntry]]
    network_id: typing.Optional[network.RequestId]
    redirected_request_id: typing.Optional[RequestId]


@dataclass
@memoize_event("Fetch.authRequired")
class AuthRequired:
    """Issued when the domain is enabled with handleAuthRequests set to true.

    The request is paused until client responds with continueWithAuth.
    """

    request_id: RequestId
    request: network.Request
    frame_id: page.FrameId
    resource_type: network.ResourceType
    auth_challenge: AuthChallenge


async def disable() -> None:
    """Disables the fetch domain.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables issuing of requestPaused events.

    A request will be paused until client calls one of failRequest, fulfillRequest or continueRequest/continueWithAuth.
    # noqa
    """
    ...


async def fail_request() -> None:
    """Causes the request to fail with specified reason.

    # noqa
    """
    ...


async def fulfill_request() -> None:
    """Provides response to the request.

    # noqa
    """
    ...


async def continue_request() -> None:
    """Continues the request, optionally modifying some of its parameters.

    # noqa
    """
    ...


async def continue_with_auth() -> None:
    """Continues a request supplying authChallengeResponse following authRequired event.

    # noqa
    """
    ...


async def continue_response() -> None:
    """Continues loading of the paused response, optionally modifying the response headers.

    If either responseCode or headers are modified, all of them must be present. # noqa
    """
    ...


async def get_response_body() -> None:
    """Causes the body of the response to be received from the server and returned as a single string.

    May only be issued for a request that is paused in the Response stage and is mutually exclusive with
    takeResponseBodyForInterceptionAsStream. Calling other methods that affect the request or disabling fetch domain
    before body is received results in an undefined behavior. # noqa
    """
    ...


async def take_response_body_as_stream() -> None:
    """Returns a handle to the stream representing the response body.

    The request must be paused in the HeadersReceived stage. Note that after this command the request can't be continued
    as is -- client either needs to cancel it or to provide the response body. The stream only supports sequential read,
    IO.read will fail if the position is specified. This method is mutually exclusive with getResponseBody. Calling
    other methods that affect the request or disabling fetch domain before body is received results in an undefined
    behavior. # noqa
    """
    ...
